/**
 * @summary     init
 * @description Display the start page that allow to load a file and select starter and substarter.
 * @version     
 * @file        init.js
 * @author      Alexan Andrieux
 * @contact     <a href="mailto:alexan.andrieux@inria.fr"> Alexan Andrieux</a>
 * @category
 *
 * @copyright 
 *
 * This source file is free software, under AGLP License
 * 
 * This source file is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the license files for details.
 * 
 * For details please refer to: <a href="http://www.gnu.org/licenses/agpl-3.0.html"></a>
 */

/**
 * Load start page that allow to load a graph file (json) or a session file (sjson).
 * If there are only one starter and substarter the visualization page is automatically loaded.
 * Else a starter and au substarter must be selected. 
 * After, the substarter has been selected the visualization page is automatically loaded. 
 * @module init
 * 
 * @requires <a href='http://blog.jquery.com/2011/09/12/jquery-1-6-4-released/'>jQuery 1.6.4</a>
 * @dependency <script language="JavaScript" type="text/javascript" src="../biojs/dependencies/jquery/jquery-1.4.2.min.js"></script>
 * 
 * @requires <a href='http://jqueryui.com/download'>jQuery UI 1.8.16</a>
 * @dependency <script language="JavaScript" type="text/javascript" src="../biojs/dependencies/jquery/jquery-ui-1.8.2.custom.min.js"></script>
 * 
 * @requires <a href='https://code.google.com/p/jquery-menubar/'>jquery.menuBar'</a>
 * 
 * @requires DataTables 
 */

$(function(){
"use strict";
	/**
	 * Build and display some elements:
	 * - Header with name and logo
	 * - Menu start page 
	 * @for buildGraph
	 * 
	 * 
	*/
	
	/**
	* And add event listener to menu elements:
	* - Select files : Load json or sjson file
	* - Clear : clear the start page
	* - Help : Load user doc page 
	* 
	* @property menuStart
	* @type object
	* @private
	*/
	var menuStart = {
		items : [{
			name : "Clear page",
			attr : {
				id : "clear_all"
			},
			selecton : function() {
				localStorage.clear();
				window.location.href = 'index.html';
				//load start page for load other graph
				//localStorage.clear('fileName');
			}
		}, {
			name : "Help",
			selecton : function() {
				window.open('doc/doc.html');
			}
		}]
	};
	
	/*Geerate Menu*/
	
	$("#menu_northPaneStartPage").menubar(menuStart)
	/* Append input file (select file everywhere on hdd) with hack for tweak the css style of this element */ 
	.children().prepend('<form id="files_upload" class="ui-state-default ui-corner-all ui-menu-title">\
							<input type="file" id="hiddenfile" class="ui-menu-title ui-state-default" style="display:none" />\
							<input type="text" id="selectedfile" class="ui-menu-title ui-state-default" />\
							<input type="button" id="upload_file" class="ui-menu-title ui-state-default" value="Select a file" readonly/>\
						</form>');
	
	$("#menu_northPaneStartPage").children(":first").addClass('flexnav one-page').attr("data-breakpoint", 800);
	
	
	$('#upload_file').click(function() {
		//TODO: add access to the resetMenu function 
		$(".menu-button").click();//close the flexnav menu
		$('#hiddenfile').click();
	});

	/* Hack for set the input file style and define action on file change*/
	$('#hiddenfile').change(function() {

		
		var files = $('#hiddenfile')[0].files;
			if(files[0] == undefined){
				alert('Error on loading file');
			} 
			var fileName = files[0].name, 
			extensionJSON = (/\.(json)$/i).test(fileName), 
			extensionSJSON = (/\.(sjson)$/i).test(fileName);
		
		
		localStorage.clear();//clear all elements in localStorage
		if($("#dataTables").hasClass("ui-tabs")){ //Destroy tabs if there are one
			$("#dataTables").tabs("destroy");
		}
		$("#dataTables").empty(); //empty div dataTables for new table
		
		if (extensionJSON) {
			localStorage.setItem("fileName", fileName);
			$('#selectedfile').val(fileName);

		} else if (extensionSJSON) {
			localStorage.setItem("sessionName", fileName);
		}
		readBlobStartPage(files);

	});
	
	$(".flexnav").flexNav();
	
	$("body").layout({
						north__paneSelector:  "#paneNorth",
						north__resizable:    false,
						north__slidable:     false,
						north__closable:     false,
	});
	/**
	* Replace alert browser with jQuery UI dialogue.
	*
	* @event alertUI
	* @param {String} message the message alert.
	*/
	window.alert = function(message) {
		$(document.createElement('div')).attr({
			title : 'Warning',
			'class' : 'alert'
		}).html(message).dialog({
			buttons : {
				OK : function() {
					$(this).dialog('close');
				}
			},
			close : function() {
				$(this).remove();
			},
			draggable : true,
			modal : true,
			resizable : false,
			width : 'auto'
		});
	};
	

	/**
	* Upload and load the selected file with test function on extension (json or sjson are allow).
	* sjson must be in ./json directory (json directory is in the root directory).
	* If correct extension then construct networkData object that defining the graph else alert message.
	* If is a sjson file visu_graph.html has been loaded
	*
	* For more details on html5 function for upload file, see  http://www.html5rocks.com/en/tutorials/file/dndfiles/
	*
	* LocalStorage use for sjson : fileName, graphData, sessionData   
	* 
	* @class readBlobStartPage
	* @constructor
	* @param {object} files FileList object with only one file
	* @return networkData
	* 
	*/
	function readBlobStartPage(files) {

		if (!files.length) {
			alert('Please select a file!');
		}
		
		var file = files[0], 
			fileName = files[0].name,
			extensionSJSON = (/\.(sjson)$/i).test(fileName),
			extensionJSON = (/\.(json)$/i).test(fileName);

		var reader = new FileReader();
		
		reader.onloadend = function(event) {
			var target = event.target;
			if (target.readyState == reader.DONE) {// DONE == 2
				
				
				if (extensionJSON) { //check json extension
					var networkData = target.result;
					initStartersTable(JSON.parse(networkData));
					return networkData;
				}else if (extensionSJSON) { //check sjson extension
					var sessionData = target.result;
					localStorage['sessionData'] = sessionData;
					
					var sessionDataParse = JSON.parse(sessionData);
					var header = sessionDataParse.header;
					var jsonfileName =  header.jsonFileName;
					var jsonGraph = sessionDataParse.graph;
					localStorage["fileName"] = jsonfileName;
					$('#selectedfile').val(jsonfileName);
					//$.getJSON("session/"+header.jsonFiles, function(json) {
						/*var substarters;
						var starters;
						$.each(json, function() {
							if (this.id ==  header.starterSelected) {
								starters = this;
								$.each(this.substarters, function() {
									if (this.data.id ==  header.substarterSelected) {
										substarters = this;
									}
								});
							}

						});*/

						localStorage.setItem('graphData',  JSON.stringify(jsonGraph));
						window.location.href = 'visu_graph.html';
						
						return jsonGraph;
					//});
				} else {
					alert('Please select JSON or SJSON files');
				}
			}else{
				alert('Error on loading file');
			};

		};

		/*reader.onloadstart = function(file) {
		 console.log("start", file)
		 };*/

		reader.onerror = function(file) {
			alert('Error on loading file');
		};

		reader.onabort = function(file) {
			alert('Loading is aborted');
		};

		//reader.readAsBinaryString(blob);
		reader.readAsText(file);
	};

	
	/**
	* Initialize, add events and display table of starters   
	* 
	* Localstorage use : graphData
	* 
	* @class initStartersTable
	* @constructor
	* @param {object} networkData define the graph.
	*  
	*/
	function initStartersTable (networkData){  
			
			var dataStarters = networkData.Starter_0, 
				tableStarters = [];  
			
			/* Compatibility with minimal json */
	        
			if (dataStarters == undefined) {
				var dataNodes = networkData.nodes, 
					dataEdges = networkData.edges, 
					nodesLen = dataNodes.length, 
					edgesLen = dataEdges.length;

				for (var i = 0; i < nodesLen; i++) {
					dataNodes[i].data.coverageValSelected = "none";
					dataNodes[i].data.coverageFileSelected = "none";
				}

				for (var j = 0; j < edgesLen; j++) {
					dataEdges[j].data.coverageValSelected = "none";
					dataEdges[j].data.coverageFileSelected = "none";
				}

				/*Open a new tab with visualization graph*/
				if (!localStorage["fromVisu"]){
					localStorage['graphData'] = JSON.stringify(networkData);
					window.location.href = 'visu_graph.html';
					//window.open('visu_graph.html');
					//$("body").load("./visu_graph.html", function(){})
				}

				return networkData;

			}else{
	         	
				var starter =[], emptyExtremGraph=[], i = 0;
													
				$("#dataTables").prepend(
					'<div id="dataTable_content" class="ui-layout-content ui-tabs ui-widget ui-widget-content ui-corner-all">\
						<div id="starters" class="content">\
							<table cellpadding="0" cellspacing="0" border="0" class="display dataGraph" id="dataGraph_starters"></table>\
						</div>\
			         </div>'
				);
				
				$.each(networkData, function() {
					starter[i] = this;
					
					if(starter[i] == undefined){
						alert('Error! Input file is not valid ');
						return false;
					} 
					
					tableStarters[i] = [this.data.id, this.data.length, this.data.sequence];
					i++;
				}); 
								
				
				/* Initialize dataTable with starters data */
				$('#dataGraph_starters').dataTable({
					"bJQueryUI" : true,
					"sPaginationType" : "full_numbers",
					"bProcessing" : true,
					"sScrollX" : "100%",
        			"bScrollCollapse": true,
					"oTableTools" : {
						"sRowSelect" : "single"
					},
					"aaData" : tableStarters,
					"aoColumns" : [{
						"sTitle" : "ID"
					}, {
						"sTitle" : "Length"
					}, {
						"sTitle" : "Sequence"
					}]

				});
				
				$("#dataGraph_starters_filter").find("input").keyup(function() {
					var highlightStr = $(this).val();
					$("#dataGraph_starters > tbody > tr").each(function() {
						var trSeq = $(this).children().get(2);
						$(trSeq).highlight(highlightStr);
		
					});
				});

				/* Define click on starter action */
				initStartersTable.prototype.onClickStarter(starter);
      }  
  	
  	}
  	
  	initStartersTable.prototype={
  	
	  	/**
		* Add events click on starters
		* 
		* 
		* @event onClickStarter
		* @param {object} starters the list of starters
		* @return {object} starter the starter selected
		*/
		onClickStarter: function (starters){
	      	
	        $('#dataGraph_starters tr').click( function(){	       				 				    	   				
	    	   	var $this = $(this);
				if ( !$this.hasClass('selected') ) {
					var i = $this.index();
					var n0 = starters[i].data.extremGraphs[0].data.nodes[0];
						
					$this.parent().find(".selected").removeClass('selected');
					$this.addClass('selected');
	    	   		
	    	   		/* add substarters tab */
	    	   		if($("#extremGraphs").get(0)==undefined && !$("#dataTables").hasClass('ui-tabs')){
	    	   			$("#dataTables").prepend('<ul>\
	    	   										<li><a id="startersTab" href="#starters">Starters</a></li>\
													<li><a id="extremGraphsRightTab" href="#extremGraphsRight">Right extensions</a></li>\
													<li><a id="extremGraphsLeftTab" href="#extremGraphsLeft">Left extensions</a></li>\
												  </ul>');
										
	    	   						
	    	   			$("#dataTable_content").append('<div id="extremGraphsRight" class="content"></div><div id="extremGraphsLeft" class="content"></div>');
						//$("#dataTable_content").append('<div id="coverageFiles" class="content"></div>');			
					};
					$("#dataTables").tabs();					
					/* add file coverage tab if coverage is present in json file */
					if(n0.data.coverage!=undefined){
						var fileSelected = n0.data.coverage[0].id,
							globalAvgCoverage=[],
							lenCoverage,
		    				avg;		
		    			
		    			//$("#dataTables ul").append('<li><a id="coverageFilesTab" href="#coverageFiles">Coverage Files</a></li>');
		    			/* define global average coverage for each coverage files */
		    			for(var l=0; l<n0.data.coverage.length; l++){
		    				lenCoverage=0;	
		    				avg =0;
			    			for(var j=0; j<starters[i].data.extremGraphs.length; j++){
			    				var extremGraphs = starters[i].data.extremGraphs[j];
			    				for(var n=0; n<extremGraphs.data.nodes.length; n++){
			    					var nodes =extremGraphs.data.nodes[n];
			    					if(nodes.data.coverage == undefined){
			    						continue;
			    					} else if(nodes.data.coverage.length == 0){
			    						continue;
			    					} else if(nodes.data.coverage[l].avg_coverage == "none"){
			    						continue;
			    					}
			    					lenCoverage++; 
			    					avg += nodes.data.coverage[l].avg_coverage;
			    					globalAvgCoverage[l] = avg;
			    				}
			    				for(var e=0; e<extremGraphs.data.edges.length; e++){
			    					var edges =extremGraphs.data.edges[e];
			    					if(edges.data.coverage == undefined){
			    						continue;
			    					} else if(edges.data.coverage.length == 0){
			    						continue;
			    					}else if(edges.data.coverage[l].avg_coverage == "none"){
			    						continue;
			    					} 
			    					lenCoverage++; 
			    					avg += edges.data.coverage[l].avg_coverage;
			    					globalAvgCoverage[l] = avg;
			    				}
		
			    			}			
			    			globalAvgCoverage[l]= (globalAvgCoverage[l]/lenCoverage).toFixed(4);
			    		}
		    						
		    			//initFilesCoverageTable(n0, globalAvgCoverage);
	    				var defaultSelectedFile = $('#dataGraph_coverageFiles tr').get(1);
	    				$(defaultSelectedFile).addClass('selected');
	    				//onClickFilesCoverage(starters[i]);
	    							
		    		}
	    			
	    			initExtremGraphsTable(starters[i], fileSelected);
	    			$("#extremGraphsRightTab").trigger("click");

	          	}else{
	          		
	          		$this.removeClass('selected');
	          		
	          	}
	          	
	          return starters[i];
	       });
          
       	}
  	};

	/**
	* Initialize, add events and display table of substarters in a tab 
	* 
	* 
	* @class initExtremGraphsTable
	* @constructor
	* @param {object} starter the starter selected
	* @param {object} starter the coverage file selected
	* 
	*/
	function initExtremGraphsTable(starter, fileSelected){
		
       	var extremGraphs = starter.data.extremGraphs;
			
       		/* Alert message if substarter have not extensions */
       		if(starter.length==0){
       			alert("Synthax problems in the input file. Graph can't be loaded.");
       			$(".ui-dialog-buttonset > button").click(function(){
						$("#startersTab").click();
				});
       			
       			return false;
       		}
       								 	
	      
		if (extremGraphs != undefined) {
			
			var nbExtremGraph = extremGraphs.length;
			var tableExtremGraphsRight= [];
			var tableExtremGraphsLeft= [];
				
       		$('#extremGraphsRight').empty()
       						 .append('<table cellpadding="0" cellspacing="0" border="0" class="display dataGraph" id="dataGraph_extremGraphsRight"></table>');
       			
       		$('#extremGraphsLeft').empty()
       						 .append('<table cellpadding="0" cellspacing="0" border="0" class="display dataGraph" id="dataGraph_extremGraphsLeft"></table>');

			
			 var inc_r =0;
			 var inc_l =0;
			
			 for(var i =0; i<nbExtremGraph; i++){
	    	   	var dataExtremGraphs = extremGraphs[i].data;
			 		if(dataExtremGraphs.direction == "RIGHT"){
					 		tableExtremGraphsRight[inc_r] = [dataExtremGraphs.id, dataExtremGraphs.direction, dataExtremGraphs.sequence, dataExtremGraphs.type];	
					 		inc_r++;
					 }else if(dataExtremGraphs.direction == "LEFT"){
					 		tableExtremGraphsLeft[inc_l] = [dataExtremGraphs.id, dataExtremGraphs.direction, dataExtremGraphs.sequence, dataExtremGraphs.type];	
					 		inc_l++;
					 }else{
			 				alert("Synthax problems in the input file");
			 		}
	
			 }
			 
		} else {
			alert("Synthax problems in the input file");

		};

	
		setTimeout(function(){ //fix header size of sequence columns
			/* Initialize dataTable with tableExtremGraphs data */ 	 
			
				$('#dataGraph_extremGraphsRight').dataTable({
					"bJQueryUI" : true,
					"sPaginationType" : "full_numbers",
					"bProcessing" : true,
					"sScrollX" : "100%",
		        	"bScrollCollapse": true,
					"oTableTools" : {
						"sRowSelect" : "single"
					},
					"aaData" : tableExtremGraphsRight,
					"aoColumns" : [{
						"sTitle" : "ID"
					}, {
						"sTitle" : "Direction"
					}, {
						"sTitle" : "Sequence"
					}, {
						"sTitle" : "Type"
					}]
		
				});
			if(inc_r>0){
				$("#dataGraph_extremGraphsRight_filter").find("input").keyup(function() {
						var highlightStr = $(this).val();
						$("#dataGraph_extremGraphsRight > tbody > tr").each(function() {
							var trSeq = $(this).children().get(2);
							$(trSeq).highlight(highlightStr);
				
					});
				});
			
				initExtremGraphsTable.prototype.onClickExtremGraphsRight(starter, extremGraphs, fileSelected);
			
				$("#extremGraphsLeftTab").trigger("click");
			}
			
			
			$('#dataGraph_extremGraphsLeft').dataTable({
					"bJQueryUI" : true,
					"sPaginationType" : "full_numbers",
					"bProcessing" : true,
					"sScrollX" : "100%",
		        	"bScrollCollapse": true,
					"oTableTools" : {
						"sRowSelect" : "single"
					},
					"aaData" : tableExtremGraphsLeft,
					"aoColumns" : [{
						"sTitle" : "ID"
					}, {
						"sTitle" : "Direction"
					}, {
						"sTitle" : "Sequence"
					}, {
						"sTitle" : "Type"
					}]
		
			});
				
			if(inc_l>0){
				$("#dataGraph_extremGraphsLeft_filter").find("input").keyup(function() {
						var highlightStr = $(this).val();
						$("#dataGraph_extremGraphsLeft > tbody > tr").each(function() {
							var trSeq = $(this).children().get(2);
							$(trSeq).highlight(highlightStr);
				
					});
				});
				
				initExtremGraphsTable.prototype.onClickExtremGraphsLeft(starter, extremGraphs, fileSelected);
				
				$("#extremGraphsRightTab").trigger("click");
			}
			
			if(inc_r==0 && inc_l==0){
				alert("No extension available. Please check your input file.");
			}
			
		}, 100);
			 

    	
	 }
	 
	initExtremGraphsTable.prototype ={
	 	/**
		* Add events click on substarters and open new tab with visu
		* 
		* 
		* @event onClickSubstarter
		* @param {object} starter the starter selected.
		* @param {object} fileSelected the coverage file selected.
		* @param {array}  extremGraphs the list of extensions graph of the starter selected
		* @return {object} networkdata the graph generated.
		*/ 
		onClickExtremGraphsRight : function (starter, extremGraphs, fileSelected){
	    	$('#dataGraph_extremGraphsRight tr').click( function(){
	    	   	var $this = $(this),
	    	    extensionLeft="none",
	    	   	extensionRight="none",
	          	i = parseInt($this.children(':first').text().replace ( /[^\d.]/g, '' ));
				
				//Remove localstorage for session when ExtremGraphs has been clicked to have correct position of nodes after load session in visu
				localStorage.removeItem("sessionName", "sessionData");
							
					if ( !$this.hasClass('selected') ) {
						$this.parent().find(".selected").removeClass('selected');
					       $this.addClass('selected'); 
					}else{
						$this.removeClass('selected');
						return false;
					}
					
					var idRight = $this.children(":first").text();
					var extremGraphsLen = extremGraphs.length;
					
					for(var i=0; i<extremGraphsLen; i++){
						if(extremGraphs[i].data.id == idRight){
							extensionRight = extremGraphs[i];
						}
					}
					
	          		$("#extremGraphsLeftTab").trigger("click");	
	          		var hasLeft= $('#dataGraph_extremGraphsLeft tr').children("td").text();
	          		if($('#dataGraph_extremGraphsLeft tr').hasClass('selected') || hasLeft=="No data available in table"){
	          			var idLeft = $('#dataGraph_extremGraphsLeft tr.selected').children(":first").text();
  
	          			for(var j=0; j<extremGraphs.length; j++){
							if(extremGraphs[j].data.id == idLeft){
								extensionLeft = extremGraphs[j];
							}
						}
	          		
	          			/* open a new tab with visualization graph */	
		    	   		var graphData = buildGraph (starter, extensionLeft, extensionRight , fileSelected);
		    	   		if(graphData!=false){
		    	   			localStorage['graphData'] = JSON.stringify(graphData);
			    	   		window.open('visu_graph.html');
			    	   		//window.location.href = 'visu_graph.html';    	   				
			          		//$("body").load("./visu_graph.html", function(){
									
								//var graphData = buildGraph (starter, extensionLeft, extensionRight /*, fileSelected*/);
							    	
			          			//localStorage['graphData'] = JSON.stringify(graphData);
			          				
			          			//return graphData;	
			          		//});
		    	   		}
		    	   		
		    	   		return graphData;
		    	   		
	          		}
	          	});
	
	       },
	       
	       /**
		* Add events click on substarters and open new tab with visu
		* 
		* 
		* @event onClickSubstarter
		* @param {object} starter the starter selected.
		* @param {object} fileSelected the coverage file selected.
		* @param {array}  extremGraphs the list of extensions graph of the starter selected
		* @return {object} networkdata the graph generated.
		*/ 
		onClickExtremGraphsLeft : function (starter, extremGraphs, fileSelected){
	    	$('#dataGraph_extremGraphsLeft tr').click( function(){
	    	   	var $this = $(this),
	    	   		extensionLeft="none",
	    	   		extensionRight="none",
	          	i = parseInt($this.children(':first').text().replace ( /[^\d.]/g, '' ));
				
				//Remove localstorage for session when ExtremGraphs has been clicked to have correct position of nodes after load session in visu
				localStorage.removeItem("sessionName", "sessionData");
							
					if ( !$this.hasClass('selected') ) {
						$this.parent().find(".selected").removeClass('selected');
					       $this.addClass('selected'); 
					}else{
						$this.removeClass('selected');
						return false;
					}
					
					var idLeft = $this.children(":first").text();
					var extremGraphsLen = extremGraphs.length;
					
					for(var i; i<extremGraphs.length; i++){
						if(extremGraphs[i].data.id == idLeft){
							extensionLeft = extremGraphs[i];
						}
					}
					   
	          		$("#extremGraphsRightTab").trigger("click");
	          		var hasRight= $('#dataGraph_extremGraphsRight tr').children("td").text();

	          		if($('#dataGraph_extremGraphsRight tr').hasClass('selected')|| hasRight=="No data available in table"){

	          			var idRight = $('#dataGraph_extremGraphsRight tr.selected').children(":first").text();
	          			
	          			for(var j=0; j<extremGraphs.length; j++){
							if(extremGraphs[j].data.id == idRight){
								extensionRight = extremGraphs[j];
							}
						}  
	          			/* open a new tab with visualization graph */
	          			var graphData = buildGraph (starter, extensionLeft, extensionRight , fileSelected);
	          			
	          			if(graphData!=false){
		    	   			localStorage['graphData'] = JSON.stringify(graphData);
			    	   		window.open('visu_graph.html');
			    	   		//window.location.href = 'visu_graph.html';    	   				
			          		//$("body").load("./visu_graph.html", function(){
									
								//var graphData = buildGraph (starter, extensionLeft, extensionRight /*, fileSelected*/);
							    	
			          			//localStorage['graphData'] = JSON.stringify(graphData);
			          				
			          			//return graphData;	
			          		//});
		    	   		}
		    	   		
		    	   		return graphData;
		         		
	          		}
	          	});
	
	       	}
	       	
	 };
	 
	 

	/**
	* Initialize, add event and display table of coverage files in a tab 
	* 
	* 
	* @class initFilesCoverageTable				substarters = starter.substarters[i];
	* @constructor
	* @param {object} n0 the first node.
	* @param {object} globalAvgCoverage average of all the avg_coverage value in this file.
	* 
	*/
	/*function initFilesCoverageTable(n0, globalAvgCoverage) {

		var arrayCoverage = n0.data.coverage, 
			tableCoverageFiles = [], 
			nbFiles = arrayCoverage.length;

		$('#coverageFiles').empty()
						   .append('<table cellpadding="0" cellspacing="0" border="0" class="display dataGraph" id="dataGraph_coverageFiles"></table>');

		for (var i = 0; i < nbFiles; i++) {
			tableCoverageFiles[i] = [arrayCoverage[i].id, globalAvgCoverage[i]];
		};
		
		$('#dataGraph_coverageFiles').dataTable({ //Initialize dataTable with substarters data
			"bJQueryUI" : true,
			"sPaginationType" : "full_numbers",
			"bProcessing" : true,
			"oTableTools" : {
				"sRowSelect" : "single"
			},
			"aaData" : tableCoverageFiles,
			"aoColumns" : [{
				"sTitle" : "Files"
			}, {
				"sTitle" : "Global avg coverage"
			}]

		});
		
	}
	
	initFilesCoverageTable.prototype = {
		/**
		* Add events click on substarters and open new tab with visu
		* 
		* 
		* @event onClickFilesCoverage
		* @param {object} starter the starter selected
		* @return {object} fileSelected the coverage file selected
		*/
		/*initFilesCoverageTable.onClickFilesCoverage = {
		onClickFilesCoverage : function(starter){
			var fileSelected ={};
			$('#dataGraph_coverageFiles tr').click( function(){
				var $this = $(this);
				if ( !$this.hasClass('selected') ) {
					$this.parent().find(".selected").removeClass('selected');
					$this.addClass('selected');
				}
					
				fileSelected = $this.children(":first").text();
					
				$('#dataGraph_substarters tr').off("click");
			 	initSubstartersTable.prototype.onClickSubstarter(starter, fileSelected);
			 		 
			 	return fileSelected;
			});		
		}
	};*/

        
    /**
	* Build graph with json data
	* 
	* 
	* @class buildGraph
	* @constructor
	* @param {object} starter the starter selected
	* @param {object} extensionLeft the left graph extension of the starter selected
	* @param {object} extensionRight the right graph extension of the starter selected
	* @param {object} fileSelected the coverage file selected
	* @return {object} graphData the graph description display in visu
	*/  
    function buildGraph (starter, extensionLeft, extensionRight, fileSelected){
       		
    	var allNodes = {},
    	   	allEdges = {}, 
    	   	graphData = {};
       
		var edgesRight =[], 
           	edgesLeft = [],
           	nodesRight = [],
           	nodesLeft = [];
           	   
		//console.log(extensionLeft, extensionRight);
		/* In case of json was corrected with phaser there are empty extension so it's necessary to test which extension is empty*/
			if(extensionRight.data != undefined){
	           	var lenNodesRight = extensionRight.data.nodes.length,
	           		lenEdgesRight = extensionRight.data.edges.length,
	           		kRight = extensionRight.data.sequence.length;
	           		
	           	extensionRight.data.k = kRight;
	           		
	          	for(var i=0; i<lenNodesRight; i++){
	          		var j =0;
	          		if(extensionRight.data.nodes[i].data.coverage != undefined && extensionRight.data.nodes[i].data.coverage.length != 0){
	          			while(extensionRight.data.nodes[i].data.coverage[j].id != fileSelected){
	          				j++;
	          			};
	          		
	          			extensionRight.data.nodes[i].data.coverageValSelected = extensionRight.data.nodes[i].data.coverage[j].avg_coverage;
	          			extensionRight.data.nodes[i].data.coverageFileSelected = extensionRight.data.nodes[i].data.coverage[j].id;
	          		}else{
	          			extensionRight.data.nodes[i].data.coverageValSelected = "none";
	          			extensionRight.data.nodes[i].data.coverageFileSelected = "none";
	          		};
	          		
	          		nodesRight.push(extensionRight.data.nodes[i]);
	          		extensionRight.data.nodes[i].data.k = kRight ;
					
	          	};
	          	
	          	
	          	for(var k=0; k<lenEdgesRight; k++){
	           		var l =0;
	          		if(extensionRight.data.edges[k].data.coverage!= undefined && extensionRight.data.edges[k].data.coverage.length != 0){
		          		while(extensionRight.data.edges[k].data.coverage[l].id != fileSelected){
		          			l++;
		          		}
		          		
		          		extensionRight.data.edges[k].data.coverageValSelected = extensionRight.data.edges[k].data.coverage[l].avg_coverage;
		          		extensionRight.data.edges[k].data.coverageFileSelected = extensionRight.data.edges[k].data.coverage[l].id;
					}else{
	          			extensionRight.data.edges[k].data.coverageValSelected = "none";
	          			extensionRight.data.edges[k].data.coverageFileSelected = "none";
	          		}
	          		
	          		edgesRight.push(extensionRight.data.edges[k]);
	          	};
          	
           };
           	
           	if(extensionLeft.data != undefined){	
	           	var lenNodesLeft = extensionLeft.data.nodes.length;
	           	var lenEdgesLeft = extensionLeft.data.edges.length;
	           	var kLeft = extensionLeft.data.sequence.length;
	           	extensionLeft.data.k = kLeft;
	           	for(var i=0; i<lenNodesLeft; i++){
	           		var j =0;
	           		if(extensionLeft.data.nodes[i].data.coverage!= undefined && extensionLeft.data.nodes[i].data.coverage.length != 0){
		           		while(extensionLeft.data.nodes[i].data.coverage[j].id != fileSelected){
		          			j++;
		          		}
		          		
		          		extensionLeft.data.nodes[i].data.coverageValSelected = extensionLeft.data.nodes[i].data.coverage[j].avg_coverage;
		          		extensionLeft.data.nodes[i].data.coverageFileSelected = extensionLeft.data.nodes[i].data.coverage[j].id;
	          		}else{
	          			extensionLeft.data.nodes[i].data.coverageValSelected = "none";
	          			extensionLeft.data.nodes[i].data.coverageFileSelected = "none";
	          		}
	          		
	          		nodesLeft.push(extensionLeft.data.nodes[i]);
	          		extensionLeft.data.nodes[i].data.k = kLeft;
	          	};
	          	
	          	for(var k=0; k<lenEdgesLeft; k++){
	           		var l =0;
	          		if(extensionLeft.data.edges[k].data.coverage!= undefined && extensionLeft.data.edges[k].data.coverage.length != 0){
		          		while(extensionLeft.data.edges[k].data.coverage[l].id != fileSelected){
		          			l++;
		          		};
		          		
		          		extensionLeft.data.edges[k].data.coverageValSelected = extensionLeft.data.edges[k].data.coverage[l].avg_coverage;
		          		extensionLeft.data.edges[k].data.coverageFileSelected = extensionLeft.data.edges[k].data.coverage[l].id;
	          		}else{
	          			extensionLeft.data.edges[k].data.coverageValSelected = "none";
	          			extensionLeft.data.edges[k].data.coverageFileSelected = "none";
	          		};
	          		
	          		edgesLeft.push(extensionLeft.data.edges[k]);
	          	};
		};
           	
		
		var testExtensionRight, testExtensionLeft;

		if(extensionLeft.data != undefined && extensionRight.data != undefined){	

        	/*Don't merge extensionRight.data.els in extensionLeft.data.els because extensionRight.data.els is kept after change starter (double edges for example)
         	 so cleaned arrays are put in several variables (edgesRight, edgesLeft,....)  
         	*/
         	 	
         	/* Generate list of nodes */
         	allNodes = $.merge(nodesRight, nodesLeft);
         	    
 			/*Generate list of edges and connect extensions with the substarter */	     			
     		allEdges = 	$.merge(edgesRight, edgesLeft);
     		
     		/* Add k to starter data*/
     		starter.data.k = kRight;
     		allNodes.push(starter);
     		
     		/*Test if extensions can be connected to the starter */
     		//if(Biojs.Sequence.prototype.validateLink(starter, extensionRight.data.nodes[0], kRight, "ff" )){
     			var rootEdgeFF = { data:{ id: "e"+allEdges.length+1, source:starter.data.id ,target: extensionRight.data.nodes[0].data.id, direction: "ff", coverage:[],coverageFileSelected:"none", coverageValSelected:"none" }};
				allEdges.push(rootEdgeFF);
				testExtensionRight = true;
     		//}else{
     			//testExtensionRight = false;
     		//};
     		//if(Biojs.Sequence.prototype.validateLink( extensionRight.data.nodes[0],starter, kRight, "rr" )){
				var rootEdgeRR = { "data":{ id: "e"+allEdges.length+1, source: extensionRight.data.nodes[0].data.id, target: starter.data.id, direction: "rr", coverage:[],coverageFileSelected:"none", coverageValSelected:"none" }};
				allEdges.push(rootEdgeRR);
				testExtensionRight = true;
     		//}else{
     			//testExtensionRight = false;
     		//};
     		
     		
			//if(Biojs.Sequence.prototype.validateLink(starter, extensionLeft.data.nodes[0], kLeft, "rf" )){
				var rootEdgeRF0 = { "data":{ id: "e"+allEdges.length+1, source:starter.data.id , target: extensionLeft.data.nodes[0].data.id, direction: "rf", coverage:[],coverageFileSelected:"none", coverageValSelected:"none" }};
				allEdges.push(rootEdgeRF0);
				testExtensionLeft = true;
     		//}else{
     			//testExtensionLeft = false;
     		//};
     		//if(Biojs.Sequence.prototype.validateLink(extensionLeft.data.nodes[0], starter, kLeft, "rf" )){
				var rootEdgeRF1 = { "data":{ id: "e"+allEdges.length+1, source: extensionLeft.data.nodes[0].data.id, target: starter.data.id, direction: "rf", coverage:[],coverageFileSelected:"none", coverageValSelected:"none" }};
				allEdges.push(rootEdgeRF1);
				testExtensionLeft = true;
     		//}else{
     			//testExtensionLeft = false;
     		//};
     		
			
			/*if(!testExtensionLeft){
				if(Biojs.Sequence.prototype.validateLink(starter, extensionLeft.data.nodes[0], kLeft, "fr" )){
     				var rootEdgeFR0 = { "data":{ id: "e"+allEdges.length+1, source:starter.data.id , target: extensionLeft.data.nodes[0].data.id, direction: "fr", coverage:[],coverageFileSelected:"none", coverageValSelected:"none" }};
					allEdges.push(rootEdgeFR0);				
					testExtensionLeft = true;
				}else{
					testExtensionLeft = false;
				}; 
				
				if(Biojs.Sequence.prototype.validateLink(extensionLeft.data.nodes[0], starter, kLeft, "fr" )){
					var rootEdgeFR1 = { "data":{ id: "e"+allEdges.length+1, source: extensionLeft.data.nodes[0].data.id, target: starter.data.id, direction: "fr", coverage:[],coverageFileSelected:"none", coverageValSelected:"none" }};
					allEdges.push(rootEdgeFR1);	
					testExtensionLeft = true;
				}else{
					testExtensionLeft = false;
				};
			}; 
     		
     		if(!testExtensionLeft && !testExtensionRight){
    			alert("Extensions defined in the input file are not valid !");
     			return false;
     		}*/		
			            
     	}else if(extensionLeft.data != undefined){

     		/* Generate list of nodes */
         	allNodes = nodesLeft;        
 				
 			/*Generate list of edges and connect extensions with the substarter */	  	
     		allEdges = 	edgesLeft;
     		
     		/* Add k to starter data*/
     		starter.data.k = kRight;
     		allNodes.push(starter);
     			
			//if(Biojs.Sequence.prototype.validateLink(starter, extensionLeft.data.nodes[0], kLeft, "rf" )){
				var rootEdgeRF0 = { "data":{ id: "e"+allEdges.length+1, source:starter.data.id , target: extensionLeft.data.nodes[0].data.id, direction: "rf", coverage:[],coverageFileSelected:"none", coverageValSelected:"none" }};
				allEdges.push(rootEdgeRF0);
				testExtensionLeft = true;
     		//}else{
     		//	testExtensionLeft = false;
     		//};
     		//if(Biojs.Sequence.prototype.validateLink(extensionLeft.data.nodes[0], starter, kLeft, "rf" )){
				var rootEdgeRF1 = { "data":{ id: "e"+allEdges.length+1, source: extensionLeft.data.nodes[0].data.id, target: starter.data.id, direction: "rf", coverage:[],coverageFileSelected:"none", coverageValSelected:"none" }};
				allEdges.push(rootEdgeRF1);
				testExtensionLeft = true;
     		//}else{
     		//	testExtensionLeft = false;
     		//};
     		
			
			/*if(!testExtensionLeft){
				if(Biojs.Sequence.prototype.validateLink(starter, extensionLeft.data.nodes[0], kLeft, "fr" )){
     				var rootEdgeFR0 = { "data":{ id: "e"+allEdges.length+1, source:starter.data.id , target: extensionLeft.data.nodes[0].data.id, direction: "fr", coverage:[],coverageFileSelected:"none", coverageValSelected:"none" }};
					allEdges.push(rootEdgeFR0);				
					testExtensionLeft = true;
				}else{
					testExtensionLeft = false;
				}; 
				
				if(Biojs.Sequence.prototype.validateLink(extensionLeft.data.nodes[0], starter, kLeft, "fr" )){
					var rootEdgeFR1 = { "data":{ id: "e"+allEdges.length+1, source: extensionLeft.data.nodes[0].data.id, target: starter.data.id, direction: "fr", coverage:[],coverageFileSelected:"none", coverageValSelected:"none" }};
					allEdges.push(rootEdgeFR1);	
					testExtensionLeft = true;
				}else{
					testExtensionLeft = false;
				};
			};*/ 
     		
     		/*if(!testExtensionLeft){
    			alert("Extensions defined in the input file are not valid !");
     			return false;
     		}*/
   
     	}else if(extensionRight.data != undefined){
     		
     		/* Generate list of nodes */	  	
         	allNodes = nodesRight;        
 			
 			/*Generate list of edges and connect extensions with the substarter */	  	
     		allEdges = 	edgesRight;
     		
     		/* Add k to starter data*/
     		starter.data.k = kRight;
     		allNodes.push(starter);
     			
	     	//if(Biojs.Sequence.prototype.validateLink(starter, extensionRight.data.nodes[0], kRight, "ff" )){
     			var rootEdgeFF = { data:{ id: "e"+allEdges.length+1, source:starter.data.id ,target: extensionRight.data.nodes[0].data.id, direction: "ff", coverage:[],coverageFileSelected:"none", coverageValSelected:"none" }};
				allEdges.push(rootEdgeFF);
				testExtensionRight = true;
     		//}else{
     			//testExtensionRight = false;
     		//};
     		//if(Biojs.Sequence.prototype.validateLink( extensionRight.data.nodes[0],starter, kRight, "rr" )){
				var rootEdgeRR = { "data":{ id: "e"+allEdges.length+1, source: extensionRight.data.nodes[0].data.id, target: starter.data.id, direction: "rr", coverage:[],coverageFileSelected:"none", coverageValSelected:"none" }};
				allEdges.push(rootEdgeRR);
				testExtensionRight = true;
     		//}else{
     			//testExtensionRight = false;
     		//};
     		
     		/*if(!testExtensionRight){
    			alert("Extensions defined in the input file are not valid !");
     			return false;
     		}*/

     	}else{
     		alert("Starter and extensions are not connected !");
     	};
     		
     		
     	/* extend substarter object with coverage attr with none value >> it's needed for datatable nodes */
		$.extend(starter.data, {
			coverage : [],
			coverageFileSelected : "none",
			coverageValSelected : "none"
		}); 


		/* Define position of extension in visu (use by layout hierarchical)*/
        if(extensionRight.data!=undefined && extensionLeft.data!=undefined){
        	graphData.position = {
        		right:extensionRight.data.nodes[0].data.id,
        		left:extensionLeft.data.nodes[0].data.id
        	};
        }else if(extensionRight.data==undefined && extensionLeft.data!=undefined){
        	graphData.position = {
        		right:undefined,
        		left:extensionLeft.data.nodes[0].data.id
        	};
        }else if(extensionRight.data!=undefined && extensionLeft.data==undefined ){
        	graphData.position = {
        		right:extensionRight.data.nodes[0].data.id,
        		left:undefined
        	};
        }else{
        	graphData.position = {
        		right:undefined,
        		left:undefined
        	};
        }
     		
        graphData.starter = starter;
        graphData.nodes = allNodes;
       	graphData.edges = allEdges;
       	
       	//console.log(graphData);     	 	
     	
    	return graphData;
       	
	}
	
});
