/**
 * @author Paul Stothard, University of Alberta, Canada
 * 
 * Add reverse complement functions to Biojs.sequence
 * @class Biojs.Sequence.extend
 * @extends Biojs.Sequence
 */

/**
 * Transform a sequence in the complement of this sequence.
 * @method complement
 * @param {String} sequence The target sequence
 * @return {String} The complement of the sequence 
 */
Biojs.Sequence.prototype.complement = function (sequence)	{ 
		var complementSequence=sequence;
		
		complementSequence = complementSequence.replace(/g/g,"1");
		complementSequence = complementSequence.replace(/c/g,"2");
		complementSequence = complementSequence.replace(/1/g,"c");
		complementSequence = complementSequence.replace(/2/g,"g");
		complementSequence = complementSequence.replace(/G/g,"1");
		complementSequence = complementSequence.replace(/C/g,"2");
		complementSequence = complementSequence.replace(/1/g,"C");
		complementSequence = complementSequence.replace(/2/g,"G");	
	
		complementSequence = complementSequence.replace(/a/g,"1");
		complementSequence = complementSequence.replace(/t/g,"2");
		complementSequence = complementSequence.replace(/1/g,"t");
		complementSequence = complementSequence.replace(/2/g,"a");
		complementSequence = complementSequence.replace(/A/g,"1");
		complementSequence = complementSequence.replace(/T/g,"2");
		complementSequence = complementSequence.replace(/1/g,"T");
		complementSequence = complementSequence.replace(/2/g,"A");
	
		complementSequence = complementSequence.replace(/u/g,"a");
		complementSequence = complementSequence.replace(/U/g,"A");
	
		complementSequence = complementSequence.replace(/r/g,"1");
		complementSequence = complementSequence.replace(/y/g,"2");
		complementSequence = complementSequence.replace(/1/g,"y");
		complementSequence = complementSequence.replace(/2/g,"r");
		complementSequence = complementSequence.replace(/R/g,"1");
		complementSequence = complementSequence.replace(/Y/g,"2");
		complementSequence = complementSequence.replace(/1/g,"Y");
		complementSequence = complementSequence.replace(/2/g,"R");	
	
		complementSequence = complementSequence.replace(/k/g,"1");
		complementSequence = complementSequence.replace(/m/g,"2");
		complementSequence = complementSequence.replace(/1/g,"m");
		complementSequence = complementSequence.replace(/2/g,"k");
		complementSequence = complementSequence.replace(/K/g,"1");
		complementSequence = complementSequence.replace(/M/g,"2");
		complementSequence = complementSequence.replace(/1/g,"M");
		complementSequence = complementSequence.replace(/2/g,"K");
	
		complementSequence = complementSequence.replace(/b/g,"1");
		complementSequence = complementSequence.replace(/v/g,"2");
		complementSequence = complementSequence.replace(/1/g,"v");
		complementSequence = complementSequence.replace(/2/g,"b");
		complementSequence = complementSequence.replace(/B/g,"1");
		complementSequence = complementSequence.replace(/V/g,"2");
		complementSequence = complementSequence.replace(/1/g,"V");
		complementSequence = complementSequence.replace(/2/g,"B");
	
		complementSequence = complementSequence.replace(/d/g,"1");
		complementSequence = complementSequence.replace(/h/g,"2");
		complementSequence = complementSequence.replace(/1/g,"h");
		complementSequence = complementSequence.replace(/2/g,"d");
		complementSequence = complementSequence.replace(/D/g,"1");
		complementSequence = complementSequence.replace(/H/g,"2");
		complementSequence = complementSequence.replace(/1/g,"H");
		complementSequence = complementSequence.replace(/2/g,"D");
			
		return complementSequence;
};


/**
 * Transform a sequence in the reverse of this sequence.
 * @method reverse
 * @param {String} sequence The target sequence
 * @return {String} The reverse of the sequence 
 */
Biojs.Sequence.prototype.reverse = function (sequence){	
	var tmpSeqArray = new Array();
	var reverseSequence = "";
	if (sequence.search(/./) != -1)	{
		tmpSeqArray = sequence.match(/./g);
		tmpSeqArray = tmpSeqArray.reverse();
		reverseSequence = tmpSeqArray.join("");
	}
	return reverseSequence;
};


/**
 * Check if concatenate sequence is valid between two nodes
 *
 * @method   validate
 * @param {object} node the current node
 * @param {object} target the target of node
 * @param {int} kmerLen the length of kmer for sequences in the graph
 * @param {String} edgeDirection the direction of the edge
 * @return {boolean} return true if concatenation is validate or false if is not
 */
Biojs.Sequence.prototype.validateLink = function(node, target, kmerLen, edgeDirection) {
		var nodeSequence;
		var targetSequence;
		
		if(typeof(node.data) === 'function'){
			nodeSequence = node.data("sequence");
		}else{
			nodeSequence = node.data.sequence;
		}
		
		if(typeof(target.data) === 'function'){
			targetSequence = target.data("sequence");
		}else{
			targetSequence = target.data.sequence;
		}
		
		
		var thisSeqLenght = nodeSequence.length;
	
		if (edgeDirection == "FF" || edgeDirection == "ff") {
			var thisSeqKmer = nodeSequence.substring(thisSeqLenght - (kmerLen - 1), thisSeqLenght);
			var targetSeqKmer = targetSequence.substring(0, kmerLen - 1);
			//console.log(thisSeqKmer, targetSeqKmer);
		} else if (edgeDirection == "RR" || edgeDirection == "rr") {
			var thisSeqKmer = this.reverse(this.complement(nodeSequence)).substring(thisSeqLenght - (kmerLen - 1), thisSeqLenght);
			var targetSeqKmer = this.reverse(this.complement(targetSequence)).substring(0, kmerLen - 1);
			//console.log(thisSeqKmer, targetSeqKmer);
		} else if (edgeDirection == "FR" || edgeDirection == "fr") {
			var thisSeqKmer = nodeSequence.substring(thisSeqLenght - (kmerLen - 1), thisSeqLenght);
			var targetSeqKmer = this.reverse(this.complement(targetSequence)).substring(0, kmerLen - 1);
			//console.log(thisSeqKmer, targetSeqKmer);
		} else if (edgeDirection == "RF" || edgeDirection == "rf") {
			var thisSeqKmer = this.reverse(this.complement(nodeSequence)).substring(thisSeqLenght - (kmerLen - 1), thisSeqLenght);
			var targetSeqKmer = targetSequence.substring(0, kmerLen - 1);
			//console.log(thisSeqKmer, targetSeqKmer);
		}
	
		if (thisSeqKmer == targetSeqKmer || this.reverse(this.complement(thisSeqKmer)) == targetSeqKmer) {
			return true;
		} else {
			return false;
		};
	
};

/**
 * Transform the title of fasta 
 * @method filterFastaTitle
 * @param {String} sequenceTitle the title of each sequence
 * @return {String} filtered title
 */
Biojs.Sequence.prototype.filterFastaTitle= function(sequenceTitle)	{
	sequenceTitle = sequenceTitle.replace(/\s{2,}/g," ");
	sequenceTitle = sequenceTitle.replace(/^\s*/g,"");
	sequenceTitle = sequenceTitle.replace(/[\f\n\r\t]+$/g,"\n");
	return sequenceTitle.replace (/[\<\>]\n/gi,"");
};

/**
 * Get the array of fasta with multi-line  
 * @method getArrayOfFasta
 * @param {String} sequenceData all the sequence enter
 * @return sequence line in array 
 */
Biojs.Sequence.prototype.getArrayOfFasta = function (sequenceData) {
	var arrayOfFasta = new Array();
	var matchArray;
	var re = /\>[^\>]+/g;
	if (sequenceData.search(/\>[^\f\n\r]+[\f\n\r]/) != -1)	{
		while (matchArray = re.exec(sequenceData)) {
			arrayOfFasta.push(matchArray[0]);
		}		
	}
	else {
		arrayOfFasta[0] = sequenceData;
	}
	return arrayOfFasta;
};

/**
 * Get only sequence of format fasta   
 * @method getSequenceFromFasta
 * @param {String} sequenceRecord the fasta lines with sequence and title
 * @return sequence without title
 */
Biojs.Sequence.prototype.getSequenceFromFasta = function (sequenceRecord) {
	if (sequenceRecord.search(/\>[^\f\n\r]+[\f\n\r]/) != -1)	{
		sequenceRecord = sequenceRecord.replace(/\>[^\f\n\r]+[\f\n\r]/,"");
	}
	return sequenceRecord;
};

/**
 * Get only title of format fasta   
 * @method getTitleFromFasta
 * @param {String} sequenceRecord the fasta lines with sequence and title
 * @return sequence without title
 */
Biojs.Sequence.prototype.getTitleFromFasta = function (sequenceRecord) {
	var fastaTitle = "Untitled";
	if (sequenceRecord.search(/\>[^\f\n\r]+[\f\n\r]/) != -1)	{
		fastaTitle = (sequenceRecord.match(/\>[^\f\n\r]+[\f\n\r]/,"")).toString();
		fastaTitle = fastaTitle.replace(/\>|[\f\n\r]/g,"");
		fastaTitle = fastaTitle.replace(/\s{2,}/g," ");
	        fastaTitle = fastaTitle.replace(/[\<\>]/gi,"");
	}
	return fastaTitle;
};

/**
 * Add return in sequence  
 * @method addReturns
 * @param {String} sequence the target sequence
 * @return sequence with return
 */
Biojs.Sequence.prototype.addReturns = function (sequence) {
	sequence = sequence.replace(/(.{60})/g, 
                    function (str, p1, offset, s) {
                      	return p1 + "\n";
                   }
        );
	return sequence;
};

/**
 * Remove white space in sequence  
 * @method removeWhiteSpace
 * @param {String} text the target sequence
 * @return sequence without whitespace
 */
Biojs.Sequence.prototype.removeWhiteSpace = function (text) {
	return text.replace(/\s/g, "");
};



