/**
 * @summary     layout.hierarchical
 * @description Add hierarchical layout to cytoscape 
 * @version     
 * @file        layout.hierarchical.js
 * @author      Alexan Andrieux
 * @contact     <a href="mailto:alexan.andrieux@inria.fr"> Alexan Andriux</a>
 * @category
 *
 * @copyright 
 *
 * This source file is free software, under AGLP License
 * 
 * This source file is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE. See the license files for details.
 * 
 * For details please refer to: <a href="http://www.gnu.org/licenses/agpl-3.0.html"></a>
 */

//TODO:getMaxLayer and getMaxOrder in graph.js to have a better scale
;(function($$) {

		/**
		 * Define default configuration of the layout:
		 * - type :	type of the layout > line or normal						  
		 * - orientation: for line type orientation horizontal or vertical											 
		 * - firPadding: value of the fit of cytoscape						
		 * - root: define root node
		 * - rightChildren: define the first right child of the root													
		 * - leftChildren: define the first left child of the root
		 * - withEdgesBends: options of graph.js to draw graph with bends
		 * - withXOrdersShift: options of graph.js to draw graph with shift on xorder node position
		 * - coefLayerDist: define the scale in y axe
		 * - coefXOrderDist: define the scale in x axe
		 * - alertSizeGraph: alert if graph is big
		 * - ready: callback on layoutready
		 * - stop: callback on layoutstop 
		 * @type {object}  
		 * @property defaults
		 * @private
		*/
	var defaults = {
		type : "line",
		orientation : "horizontal",
		fitPadding: 100, // padding when fitting
		root:undefined,
		boundingBox: undefined,
		rightChildren:undefined,
		leftChildren: undefined,
		withEdgesBends : true,
		withXOrdersShift : true,
		coefLayerDist: [0,100], // define distance between two nodes by coefLayerDist[0]+coefLayerDist[1]*node.layer
		coefXOrderDist: [0,100], // define distance between two nodes by coefXOrderDist[0]+coefXOrderDist[1]*node.xorder
		alertSizeGraph: 250,
		ready: undefined, // callback on layoutready
		stop: undefined // callback on layoutstop
	};
	
	
	/**
	 * Add hierarchical layout to cytoscape layout
	 * @class HierarchicalLayout
	 * @param {object} options the configuration for the layout
	 * @constructor
	 */
	function HierarchicalLayout(options) {
		this.options = $$.util.extend({}, defaults, options);
	}
	
	/**
	 * Draw the layout with graph.js library
	 * @method drawLayout
 	 * @param {object} options the configuration for the layout
 	 * @private
 	 */
	function drawLayout(options){
	var startTime = new Date().getTime();
	  
    var elapsedTime = 0;  

		var cy = options.cy,
			edges = cy.edges(),
			graph = new Graph();
			console.log(options);
		
	 var nodes = cy.nodes().not(':parent');
	 /*options.boundingBox = $$.util.makeBoundingBox( options.boundingBox ? options.boundingBox : {
      x1: 0, y1: 0, w: cy.width(), h: cy.height()
    });*/
		//initialize position to 0 >> fix bug nodes without edges are locked
		var p=0;
		cy.nodes().each(function(){
			
			this.positions(function(){
				return {
					x:  options.coefXOrderDist[1]*p,
					y: -2*options.coefLayerDist[1]
				};
			});
			p++;
		});
		
		var childrenPosition = {
									right: options.rightChildren,
									left: options.leftChildren
							};
							
		edges.each(function() {
			if (this.data("direction") == "FF" || this.data("direction") == "ff") {
				graph.addEdge(this.data("source"), this.data("target"));
			} else if ((this.data("direction") == "rf" || this.data("direction") == "RF") && this.data("source") < this.data("target")) {
				graph.addEdge(this.data("source"), this.data("target"));
			}else if ((this.data("direction") == "fr" || this.data("direction") == "FR") && this.data("source") > this.data("target")) {
				graph.addEdge(this.data("source"), this.data("target"));
			}

		});
		
		graph.hierarchicalDrawing(options.orientation, options.type, options.withEdgesBends, options.withXOrdersShift, options.root, childrenPosition);
		//console.log(graph.nodes)
		var coefLayerDist = options.coefLayerDist,
			coefXOrderDist = options.coefXOrderDist,
			graphNodes= graph.nodes;
		
		for (var n in graph.nodes) {
			var cyNode = cy.elements("node[id='" + graphNodes[n].id + "']");
			cyNode.position('y', coefLayerDist[0] + coefLayerDist[1] * graphNodes[n].layer);
			cyNode.position('x', coefXOrderDist[0] + coefXOrderDist[1] * graphNodes[n].xOrder);
		}
		
		 elapsedTime = new Date().getTime() - startTime;
		 var minutes =  elapsedTime/(1000*60);
		 var seconds = elapsedTime/1000;
    	 console.log(Math.floor(minutes) + "min "+Math.floor((minutes-Math.floor(minutes))*60)+ "sec "+Math.floor(((seconds)-Math.floor(seconds))*1000)+" ms");  

	}

	/**
	 * Run the layout
	 * @method run
	 * 
	 */
	HierarchicalLayout.prototype.run = function() {
		var options = this.options,
			cy = options.cy,
			nodes = cy.nodes(),
			edges = cy.edges(),
			container = cy.container(),
			width = container.clientWidth,
			height = container.clientHeight;

		/*
		 * test of graph.js
		 *
		 */
		
		// arbor doesn't work with just 1 node
		if(nodes.length <= 1 ){
			if( options.fit ){
				cy.reset();
			}

			cy.one("layoutstop", options.stop);
			cy.trigger("layoutstop");

			cy.one("layoutstop", options.stop);
			cy.trigger("layoutstop");

			return;
		}
		
		options.ready = false;
		
		var closed = true;
		if (edges.length > options.alertSizeGraph) {
			closed = false;
			$("#dialog-confirm").dialog({
				resizable : false,
				buttons : {
					"Continue" : function() {
						var $this = $(this);
						$this.dialog("close");
						var layoutSelected = $this.find("#layoutSelection").val();
						setTimeout(function(){
							closed= true;
							if(layoutSelected != "hierarchical"){
									cy.layout({name:  layoutSelected});
									return;
							}else{
									drawLayout(options);
									if( !options.ready && closed){
										options.ready = true;
										cy.one("layoutready", options.ready);
										cy.trigger("layoutready");
									}
									
									cy.one("layoutstop", options.stop);
									cy.trigger("layoutstop");
									
									if( cy.elements().size() === 0 ){
											cy.reset();
									} else {
											cy.fit( options.fitPadding );
									}	
									return;								
							}
						},10);
					}
				}
			});
			$("#dialog-confirm").find(".ui-icon-alert").after("<div id=textAlert>The graph has " + cy.nodes().length + " nodes and " + cy.edges().length + " edges. Generate hierarchical layout increase the time to load the graph. </div>");
			$("#dialog-confirm").find("#textAlert").after(
				"<label class=layoutLabels for=layout>Layout :</label>\
					<select id=layoutSelection type=text>\
					  <option value=hierarchical selected=selected>Hierarchical</option>\
					  <option value=grid>Grid</option>\
					  <option value=random>Random</option>\
					  <option value=arbor>Arbor</option>\
					  <option value=circle>Circle</option>\
					  <option value=cose>Cose</option>\
					  <option value=breadthfirst>Breadthfirst</option>\
					  <option value=null>Null</option>\
					</select>");
					
		} else {
			drawLayout(options);
		}
		
			if( !options.ready && closed){
						options.ready = true;
						cy.one("layoutready", options.ready);
						cy.trigger("layoutready");
			}
			
			cy.one("layoutstop", options.stop);
			cy.trigger("layoutstop");
			
			if( cy.elements().size() === 0 ){
					cy.reset();
			} else {
					cy.fit( options.fitPadding );
			}
		
		
		
	};

	HierarchicalLayout.prototype.stop = function() {
		
	};
	
	//add layout to cytoscape layout list
	$$("layout", "hierarchical", HierarchicalLayout);

})(cytoscape);
