/*
*
*  (c) 2013 Philipp Strathausen <strathausen@gmail.com>, http://strathausen.eu
*  Contributions by Jake Stothard <stothardj@gmail.com>.
*
*  based on Dracula Graph Layout and Drawing Framework 0.0.3alpha
*  (c) 2010 Philipp Strathausen <strathausen@gmail.com>, http://strathausen.eu
*  Contributions by Jake Stothard <stothardj@gmail.com>.
*
*  based on the Graph JavaScript framework, version 0.0.1
*  (c) 2006 Aslak Hellesoy <aslak.hellesoy@gmail.com>
*  (c) 2006 Dave Hoover <dave.hoover@gmail.com>
*
*  Ported from Graph::Layouter::Spring in
*    http://search.cpan.org/~pasky/Graph-Layderer-0.02/
*  The algorithm is based on a spring-style layouter of a Java-based social
*  network tracker PieSpy written by Paul Mutton <paul@jibble.org>.
*
*  This code is freely distributable under the MIT license. Commercial use is
*  hereby granted without any cost or restriction.
*
*  Links:
*
*  Graph Dracula JavaScript Framework:
*      http://graphdracula.net
*
/*--------------------------------------------------------------------------*/

/**
 A node is defined by its id. The id is unique.
**/

var AbstractEdge = function() {
};
AbstractEdge.prototype = {
	hide : function() {
		this.connection.fg.hide();
		this.connection.bg && this.bg.connection.hide();
	}
};
var EdgeFactory = function() {
	this.template = new AbstractEdge();
	this.template.style = new Object();
	this.template.style.directed = false;
	this.template.weight = 1;
};
EdgeFactory.prototype = {
	build : function(source, target) {
		var e = jQuery.extend(true, {}, this.template);
		e.source = source;
		e.target = target;
		return e;
	}
};

/*
 * Graph
 */
var Graph = function() {
	this.nodes = {};
	//this.nodes =new Array();
	this.edges = [];
	//this.edges = new Array();
	this.edgeFactory = new EdgeFactory();
};

Graph.prototype = {
	/*
	 * add a node
	 * @id          the node's ID (string or number)
	 * @content     (optional, dictionary) can contain any information that is
	 *              being interpreted by the layout algorithm or the graph
	 *              representation
	 */
	addNode : function(id, content) {
		/* testing if node is already existing in the graph */
		
		var newNode = this.nodes[id];
		if (newNode == undefined) {
			newNode = new Graph.Node(id, content);
			this.nodes[id]= newNode;
		}
		
		return newNode;
	},

	addEdge : function(source, target, style) {
		var s = this.addNode(source);
		var t = this.addNode(target);
		var edge = this.edgeFactory.build(s, t);
		jQuery.extend(edge.style, style);
		s.edges.push(edge);
		this.edges.push(edge);
		// NOTE: Even directed edges are added to both nodes.
		t.edges.push(edge);
	},

	/**
	copy in a new graph the current one
	**/
	copy : function() {
		var graph = new Graph();
		var lenEdges= this.edges.length;
		
		for(var e=0; e<lenEdges; e++){
			var edge = this.edges[e];
			graph.addEdge(edge.source.id, edge.target.id);
		}
		
		for (var n in this.nodes) {
			if (this.nodes[n].layer != undefined)
				graph.nodes[n].layer = this.nodes[n].layer;
		}

		return graph;
	},

	/**
	 Under construction, to be checked !!!
	 **/
	removeEdge : function(edge) {
		//var arrayNodes = jQuery.makeArray(this.nodes);
		//console.log(this.nodes, arrayNodes)
		$.each(this.nodes, function(){
			for(var e=0; e<this.edges.length; e++){
				var eToTest = this.edges[e];
				if (eToTest.source.id == edge.source.id && eToTest.target.id == edge.target.id)
					this.edges.splice(e, 1);
			}
		});
		var edgeIndex = this.edges.indexOf(edge);
		this.edges.splice(edgeIndex, 1);
	},
	/**
	 Given a set of edges, remove them from the graph
	 **/
	removeEdges : function(edges) {
		var len = edges.length;
		for(var e=0; e<len; e++){
			this.removeEdge(edges[e]);
		}
	},

	/**
	 Remove a node of the graph
	 **/
	removeNode : function(node) {
		var edgesToRemove = [];
		for (var x = 0; x < node.edges.length; x++) {
			var edge = node.edges[x];
			edgesToRemove.push(edge);
		}
		var lenRmovedEdges = edgesToRemove.length;
		for(var e=0; e<lenRmovedEdges; e++){
			this.removeEdge(edgesToRemove[e]);
		}
		delete this.nodes[node.id];
	},

	/**
	 Return the leafs of the graph
	 **/
	getLeafs : function() {
		var leafs = new Array();
		var self = this;
		$.each(this.nodes, function(){
			if (self.nodeOutDegree(this) == 0)
				leafs.push(this);
		});
		return leafs;
	},
	
	/**
	Returns the inDegree in the graph structure
	**/
	nodeInDegree : function(node) {
		var inDegree = 0;
		var len = node.edges.length;
		for(var e=0; e<len; e++){
			var other = node.edges[e].source;
			if (other.id != node.id)
				inDegree++;
		}
		return inDegree;
	},

	/**
	 Returns the outDegree in the graph structure
	**/
	nodeOutDegree : function(node) {
		var outDegree = 0;
		var len = node.edges.length;
		for(var e=0; e<len; e++){
			//var other = node.edges[e].source;
			//if (other.id==node.id)
			var other = node.edges[e].target;
			if (other.id != node.id)
				outDegree++;
		}
		return outDegree;
	},

	/***
	 This method is based on the Sugiyama framework as described in Healy and Nikolov chapter published in 2013
	 Hierarchical drawing steps:
	 1 CYCLE REMOVAL: ok using getMinimalFAS edges and removing them.
	 2 LAYER ASSIGNMENT ("note that section 13.3.4 Layer-Assignment with Long Vertices" describes how to work with different sizes of nodes, but it's hard then to use the following optimisations: step 3 4 and 5)
	 3 Edge Concentration (mentioned to be optional) for the moment I skip this one
	 4 Vertex Ordering
	 5 x-Coordinate Assignment
	 
	 Optional drawing steps: 
	 6 Linearize 
	 7 graph orientation
	 8 minimize crossings and superposed edges
	 9 if graph line test the correct position of each children root
	 ***/
	//TODO: use root when it's define in options of layout.hierarchical.js
	hierarchicalDrawing : function(orientation, type, bends, shift, rootId, childrenPosition) {

		//1 CYCLE REMOVAL: ok using getMinimalFAS edges and removing them.
		var dag = this.extractDAG();
		
		var removeEdgesFromDag = this.getMinimalFAS();
		var root = this.getRoot(rootId);
		//2 LAYER ASSIGNMENT
		dag.layerAssignment();
		
		//3 EDGE CONCENTRATION (mentioned to be optional) for the moment I skip this one

		//4 VERTEX ORDERING. Pb to fix: last (top) layer may not be welled ordered.
		//dag.createGraphWithBends();
		if (bends) {
			dag.createGraphWithBends();
		}
		dag.vertexOrdering(root);
		
		//6 Linearize Graph if is reqiered
		if (type == "line") {
			dag.linearize(removeEdgesFromDag, root);
		}
		
		//5 X-COORDINATE ASSIGNMENT
		if (shift) {		
			dag.xOrderShift(type, root);	
		}


		//7 Put the computed information into the graph and define orientation
		for (n in this.nodes) {			
			if (orientation == "vertical") {
				this.nodes[n].layer = dag.nodes[n].layer;
				this.nodes[n].xOrder = dag.nodes[n].xOrder;
			} else if (orientation == "horizontal") {
				this.nodes[n].layer = dag.nodes[n].xOrder;
				this.nodes[n].xOrder = dag.nodes[n].layer;
			}
		}
		
		//8 Post algo to minimize edges crossings and superposed nodes and edges
		//do{
			//console.log("number of crossings "+this.getTotalCrossings());
			//console.log("number of superposed edges "+this.getTotalSuperposedEdges());
			var saveNumberCrossings = this.getTotalCrossings();
			var saveNumberSuperposed = this.getTotalSuperposedEdges();
			
			//minimise cronssings edges function
			this.minimizeSuperposed(root);
			
			//minimize suerposed Nodes on edges or others nodes
			this.minimizeCrossings(type, root);

			//if((saveNumberCrossings==this.getTotalCrossings() && saveNumberSuperposed==this.getTotalSuperposedEdges())
			//||(this.getTotalCrossings()==0 && this.getTotalSuperposedEdges()==0)){
				//break;
			//}
		//}while(this.getTotalCrossings()!=0 && this.getTotalSuperposedEdges()!=0);
		
		//console.log("number of crossings "+this.getTotalCrossings());
		//console.log("number of superposed edges "+this.getTotalSuperposedEdges());
		
		//9 Test if position of leaf right and leaf left is correct and set if it's not when is line graph
		if (type == "line") {
			var childrenRoot = this.getOutNodesFrom(root);
			var lenchildrenRoot= childrenRoot.length;
			for (var c=0; c<lenchildrenRoot; c++) {
				if (orientation == "vertical" && (childrenPosition.left!="" || childrenPosition.right!="") && (childrenPosition.left!=undefined || childrenPosition.right!=undefined)) {
					if (childrenRoot[c].layer < root.layer && childrenPosition.left != childrenRoot[c].id) {
						this.reverseLayer();
					}else if (childrenRoot[c].layer > root.layer && childrenPosition.right != childrenRoot[c].id) {
						this.reverseLayer();
					}
				} else if (orientation == "horizontal" && (childrenPosition.left!="" || childrenPosition.right!="") && (childrenPosition.left!=undefined || childrenPosition.right!=undefined)) {
					if (childrenRoot[c].xOrder < root.xOrder && childrenPosition.left != childrenRoot[c].id) {
						this.reverseXOrder();
					}else if (childrenRoot[c].xOrder > root.xOrder && childrenPosition.right != childrenRoot[c].id) {
						this.reverseXOrder();
					}
				}
			}
		}
			
		
		/*for(var k in this.nodes){
				var superposedNodes = this.getNodesSuperposedOnEdges(this.nodes[k]);
				console.log(superposedNodes)
				//for(var l in superposedNodes){
					//superposedNodes[l].layer++;
				//}
		}*/
	},

	/** 
	 reverse vertical of the graph the top become the bottom and the bottom become the top
	**/
	reverseLayer : function() {
		var layerMax = 0;
		$.each(this.nodes, function(){
			if (layerMax < this.layer) {
				layerMax = this.layer;
			}
		});
		$.each(this.nodes, function(){
			this.layer = layerMax - this.layer;
		});
	},

	/** 
	 reverse horizontal of the graph the right become the left and the left become the right
	**/
	reverseXOrder : function() {
		var xOrderMax = 0;
		$.each(this.nodes, function(){
			if (xOrderMax < this.xOrder) {
				xOrderMax = this.xOrder;
			}
		});
			$.each(this.nodes, function(){
			this.xOrder = xOrderMax - this.xOrder;
		});
	},

	/**
	 find if the graph have a node root
	**/
	getRoot : function(rootId) {
		var root;
		var self =this;
		if(rootId != undefined){
			$.each(this.nodes, function(){
				if (this.id == rootId) {
					
					root = this;
				}
			});
		}else{
			$.each(this.nodes, function(){
				if (self.nodeInDegree(this) == 0 && self.nodeOutDegree(this) == 2) {
				
					root = this;
				}
			});
		}
	//console.log(root);		
	return root;
	},
	
	/** 
	 find the max absolute value in nodes layer
	**/
	getAbsMaxLayer :function(){
		var extremLayer = 0;
		$.each(this.nodes, function(){
				if(Math.abs(this.layer)>extremLayer){
						extremLayer=Math.abs(this.layer);
				}
					
		});
		return extremLayer;
	},
	
	/**
	 find the max absolute value in nodes xorder
	**/
	getAbsMaxXOrder :function(){
		var extremXOrder = 0;
		$.each(this.nodes, function(){
				var u = this.nodes[n];
				
				if(Math.abs(this.layer)>extremXOrder ){
						extremXOrder =Math.abs(this.layer);
				}
					
		});
		return extremXOrder;
		
	},

	/*split graph in to part when there are a root */
	splitGraph : function() {
		//TODO: do with dfs function
	},

	/**
	  xOrderShift is necessary to linearize graph in some cases.
	  If type is line:
		  1  Put the root next to the right child if necessay for construct the first part of the tree
		  2  For each part of graph, move nodes on each side of the child of the root
		  3  The second part of the tree is put under the first one  
	**/
	xOrderShift : function(type, root) {

		var layers = [];
		var rootLayer = root.layer;
		var rootXOrder = root.xOrder;
		
		if(type == "line"){
			
			//calculate Max xOrder and layer
			/*var MaxLayerSize = 0;
			var MaxXOrderSize = 0;
			$.each(this.nodes, function(){
				if (MaxLayerSize < this.layer)
					MaxLayerSize = this.layer;
					
				if ( MaxXOrderSize < this.xOrder)
					 MaxXOrderSize = this.xOrder;
			});
			
			
			
			var initialRootLayer = root.layer;*/
			
			var self=this;
			$.each(this.nodes, function(){
				//2 move nodes on each side of the child of the root
				self.moveSuperposeEdges(this);
				
				//3 force to put the second graph part under the first one
				/*if (this.layer > root.layer+1 && this.id != root.id) {
					this.xOrder= this.xOrder + 2*MaxXOrderSize;
					this.layer = root.layer - (initialRootLayer - this.layer);
				}*/
				
				if(this.id == root.id){
					root = this;
				}
				
			});
			
			
		}else{
			//TODO: validate this
			$.each(this.nodes, function(){
				var layer = this.layer;
				if (layers[layer] == undefined)
					layers[layer] = new Array();
				layers[layer].push(this);

			});

			var MaxLayerSize = 0;
			var lenLayers= layers.length;
			for (var l=0; l<lenLayers; l++) {
				if (MaxLayerSize < layers[l].length)
					MaxLayerSize = layers[l].length;
			}
		
			for (var i = 0; i < lenLayers; i++) {
				if (layers[i] != undefined) {
					var currentLayerSize = layers[i].length;
					for(var n=0; n<currentLayerSize; n++){
						var u = layers[i][n];
	
						if (root.id != undefined) {
							//optimize for graph with root
							if (u.id == root.id) {
								 //=======>>>>>>> .theirs
								 u.xOrder=u.xOrder+Math.floor(MaxLayerSize);
							}
	
							//for other graph
						} else {
							u.xOrder = u.xOrder + Math.floor(MaxLayerSize / currentLayerSize);
						}
	
					}
	
				}
			}
		}
		
		// for(var i=1;i<layers.length;i++)
		// {
		// 	for(n in layers[i])
		// 	{
		// 		var u=layers[i][n];
		// 		var nEdges=0;
		// 		var sumPos=0;
		// 		for(e in u.edges)
		// 		{
		// 			if(u.edges[e].source.id==u.id)
		// 			{
		// 				var v=u.edges[e].target;
		// 			}
		// 			else
		// 			{
		// 				var v=u.edges[e].source;
		// 			}
		// 			if(v.layer==i-1)
		// 			{
		// 				nEdges++;
		// 				sumPos=sumPos+v.xOrder;
		// 			}
		// 		}
		// 		if(nEdges!=0)
		// 			u.xOrder=Math.floor(sumPos/nEdges);
		// 	}
		//}
	},


	/**Add bend nodes to a layered graph
	 for each edge,if the layer of the source and the target have a difference more than 1
	 Then add bends
	 will return the new graph with bend nodes and corresponding edges
	**/
	createGraphWithBends : function() {
		//var cDag=this.copy();
		var len= this.edges;
		for (var e=0; e<len; e++) {
			if (Math.abs(this.edges[e].source.layer - this.edges[e].target.layer) > 1) {
				//console.log("---- Add bend nodes for edge "+this.edges[e].source.id+","+this.edges[e].target);
				this.addBendNodes(this.edges[e]);
			}
		};
	},

	/**
	 add bends
	 **/
	addBendNodes : function(edge) {
		console.log("---- Add bend nodes for edge " + edge.source.id + "," + edge.target.id);
		//add the bend node and corresponding edges
		//for edge (u,v) (layer u is necesseraly before layer v)
		//	currentNode=u
		//	for (l=layer(u)-1 and l>layer(v); l--)
		//		add node b "b+u+v+l"
		//		set node.layer=l
		//		add edge (currentNode,b)
		//		currentNode=b
		//	add the final edge (currentNode,v)
		var u = edge.source;
		var v = edge.target;
		var currentNode = u;
		for (var l = u.layer - 1; l > v.layer; l--) {
			console.log("Add node " + "b_" + u.id + "-" + v.id + "_" + l);
			var b = this.addNode("b_" + u.id + "-" + v.id + "_" + l);
			b.layer = l;
			this.addEdge(currentNode.id, b.id);
			currentNode = b;
		};
		this.addEdge(currentNode.id, v.id);
	},

	/**
	 for each layer, will create an ordering of its nodes
	**/
	vertexOrdering : function(root){
		//var childrenRoot = this.getOutNodesFrom(root);
		//layers are going from 0 to x
		var layers = [];
		var self = this;
		$.each(this.nodes, function(){
			var layer = this.layer;
			this.xOrder = 0;
			root.xOrder = 0;
			root.layer = layer;
			if (layers[layer] == undefined)
				layers[layer] = new Array();
			layers[layer].push(this);
		});
		//set a random xOrder for each node per layer
		var lenLayers = layers.length;
		var saveLayerVal = 0;
		var rootLayerVal = 0;
		var maxLayerVal = this.getAbsMaxLayer();
		for (var l=0; l<lenLayers; l++){
			var layer = layers[l];
			var xOrder = 0;
			for (n in layer) {

				layer[n].xOrder = xOrder;
				
				if(layer[n].id == root.id){
					root.xOrder = xOrder;
				}
				
				xOrder++;
				
				
				if(layer[n].layer - saveLayerVal > 1){
						layer[n].layer = maxLayerVal - layer[n].layer + saveLayerVal+1;
				}else{
						saveLayerVal = layer[n].layer;		
				}
				
				if(layer[n].id == root.id){
					root.layer = layer[n].layer;
				}
				
			}
			
		}
		
		$.each(layers, function(){				
			//console.log("before "+this.getTotalCrossings());
			self.greedySwitch(this);
		});
		
		///console.log("after "+this.getTotalCrossings());
	},
	
	/**
	greedySwitch algo:
		Create he crossing matrix
		Use a heuristic to improve the overall crossing
		Use the median method to determine the initial ordering
		Use an adjacent exchange method to refine
	**/
	greedySwitch : function(layer) {
		var numberOfCrossings = 0;
		if (layer.length == 1)
			return numberOfCrossings;
		else {
			/*	for(var i=0;i<layer.length;i++)
			 {

			 for(var j=i+1;j<layer.length;j++)
			 {
			 numberOfCrossings=numberOfCrossings+this.getCrossingNumber(layer[i],layer[j]);
			 }

			 }*/
			
			//numberOfCrossings = this.getTotalCrossings();
			var crossingBeforeRun = numberOfCrossings;
			var crossingAfterRun = 0;
			var xOrderMax = 0;
			console.log("before " + crossingBeforeRun);
			while (crossingBeforeRun != crossingAfterRun) {
				crossingBeforeRun = crossingAfterRun;
				crossingAfterRun = 0;
				for (var i = 0; i < layer.length; i++) {
					var u = layer[i];
					for (var j = i + 1; j < layer.length; j++) {
						var v = layer[j];
						nbij = this.getCrossingNumber(u, v);
						u.xOrder = j;
						v.xOrder = i;
						nbji = this.getCrossingNumber(u, v);
						if (nbji < nbij) {
							layer[i] = v;
							layer[j] = u;
							crossingAfterRun = crossingAfterRun + nbji;
							console.log("swap " + u.id + " and " + v.id);
						} else {
							u.xOrder = i;
							v.xOrder = j;
							crossingAfterRun = crossingAfterRun + nbij;
						}
					}
				}
			}
			
			
			/*if(layer.length>1){
				for (n in layer) {
					if(n%2==0){
							layer[n].xOrder= -layer[n].xOrder+1;
					}else{
		
					}
				
				}
			}*/
			//console.log("--"+numberOfCrossings);
			return numberOfCrossings;
		}

	},
	
	/**
	total number of crossings edges in the graph except superposed edges
	**/
	getTotalCrossings : function() {
		var total = 0;
		for (n1 in this.nodes) {
			var u = this.nodes[n1];
			//console.log("u "+u.id);
			for (n2 in this.nodes) {
				var v = this.nodes[n2];
				//console.log("v "+v.id);
				//console.log(this.getCrossingNumber(u,v));
				total = total + this.getCrossingNumber(u, v);
			}
		}
		return total;
	},
	
	/**
	 number of crossings that edges incident to u make with edges incident v, when x1(u)<x1(v)
	 number of pairs (u,w) and (v,z) of edges with x2(z)<x2(w)
	 superposed edges are not fund with this function
	 **/
	getCrossingNumber : function(u, v) {
		var crossNumber = 0;
		//var source = this.getOutNodesFrom(u);
		if ((u.xOrder < v.xOrder) && (u.layer == v.layer) && (v.id!=u.id)) {
			var lenUedges = u.edges.length;
			for(var e1=0; e1<lenUedges; e1++){
				var w;
				if (u.edges[e1].source.id == u.id)
					w = u.edges[e1].target;
				else
					w = u.edges[e1].source;
					
					var lenVedges = v.edges.length;
					for(var e2=0; e2<lenVedges; e2++){
					var z;
					if (v.edges[e2].source.id == v.id)
						z = v.edges[e2].target;
					else
						z = v.edges[e2].source;
					//if((w.layer==z.layer==u.layer+1)&&(z.xOrder<w.xOrder))
					if ((w.layer > u.layer) && (z.layer > u.layer) && (z.xOrder < w.xOrder)) {
						crossNumber++;
					}

				}
			}
		}
		
		if ((v.xOrder < u.xOrder) && (u.layer == v.layer) && (v.id!=u.id)) {
				var lenVedges = v.edges.length;
				for(var e1=0; e1<lenVedges; e1++){
				var w;
				if (v.edges[e1].source.id == u.id)
					w = v.edges[e1].target;
				else
					w = v.edges[e1].source;
				
				var lenUedges = u.edges.length;
				for(var e2=0; e2<lenUedges; e2++){
					var z;
					if (u.edges[e2].source.id == u.id)
						z = u.edges[e2].target;
					else
						z = u.edges[e2].source;
					//if((w.layer==z.layer==u.layer+1)&&(z.xOrder<w.xOrder)){
					if ((w.layer > u.layer) && (z.layer > u.layer) && (z.xOrder < w.xOrder)) {
						crossNumber++;
						//console.log(w.id,v.id,z.id);
					}

				}
			}
		}
		
		/*for(var i=0; i<this.edges.length; i++){
			var edge1 = this.edges[i];
			for(var j=0; j<this.edges.length; j++){
				var edge2 = this.edges[j];

				if(edge1!=edge2 && 
					edge1.source!=edge2.target &&  
					edge1.target!=edge2.source && 
					edge1.source.xOrder>edge2.source.xOrder && 
					edge1.target.xOrder<edge2.target.xOrder){
					crossNumber++;

				}
						
			
			}*/
			
			/*Parcours arrayW
			 * 	Parcours arrayZ
			 * 		test si w.edges[i].id == z.id
			 * 		si différent
			 * 			regarder w.layer et z.layer // w.xorder et z.xorder
			 * 			si z.layer< w.layer && z.xorder > w.xorder
			 * 				alors croisement
			 * 			
			 * 			
			 */
		return crossNumber;
	},

	/**
	return nodes with crossings edges or false if no crossings edges 
	**/
	//TODO: more deep detection for edges crossings
	getNodesWithCrossingEdges : function(u, v) {
		var nodesWithCrossingEdges = [];
		if ((u.xOrder < v.xOrder) && (u.layer == v.layer)) {
			var lenUedges = u.edges.length;
			for(var e1=0; e1<lenUedges; e1++){
				var w;
				if (u.edges[e1].source.id == u.id)
					w = u.edges[e1].target;
				else
					w = u.edges[e1].source;
					
				var lenVedges = v.edges.length;
				for(var e2=0; e2<lenVedges; e2++){
					var z;
					if (v.edges[e2].source.id == v.id)
						z = v.edges[e2].target;
					else
						z = v.edges[e2].source;
					//if((w.layer==z.layer==u.layer+1)&&(z.xOrder<w.xOrder)){
					if ((w.layer > u.layer) && (z.layer > u.layer) && (z.xOrder < w.xOrder)) {
						nodesWithCrossingEdges = [w, u, z];
						//console.log(w.id,u.id,z.id);
					}

				}
			}
		} else if ((v.xOrder < u.xOrder) && (u.layer == v.layer)) {
			var lenVedges = v.edges.length;
			for(var e1=0; e1<lenVedges; e1++){
				var w;
				if (v.edges[e1].source.id == u.id)
					w = v.edges[e1].target;
				else
					w = v.edges[e1].source;
					
				var lenUedges = u.edges.length;
				for(var e2=0; e2<lenUedges; e2++){
					var z;
					if (u.edges[e2].source.id == u.id)
						z = u.edges[e2].target;
					else
						z = u.edges[e2].source;
					//if((w.layer==z.layer==u.layer+1)&&(z.xOrder<w.xOrder)){
					if ((w.layer > u.layer) && (z.layer > u.layer) && (z.xOrder < w.xOrder)) {
						nodesWithCrossingEdges = [w, u, z];
						//console.log(w.id,v.id,z.id);
					}

				}
			}
		}
		if(nodesWithCrossingEdges.length!=0){
			return nodesWithCrossingEdges;
		}else{
			return false;
		}
		
	},
	
	/**
	total number of edges superposed 
	**/
	getTotalSuperposedEdges : function() {
		var total = 0;
		var self =this;
		$.each(this.nodes, function(){
			total = total + self.getSuperposedEdgesNumber(this);
		});
		return total;
	},
	
	/**
	number of edges superposed per node 
	**/
	getSuperposedEdgesNumber:function(u){

		var target = this.getInNodesTo(u);
		//var superposedNodes = [];
		var superposedEdgesNumber = 0;

		if (target.length > 1) {
			for (var j = 0; j < target.length; j++) {
					var dist = Math.abs(target[j].xOrder - u.xOrder);
					if(dist>=1){
						for(var k=1; k<dist; k++){
							$.each(this.nodes, function(){
								if (this.layer == u.layer && 
									this.layer == target[j].layer && 
									this.id != target[j].id && 
									this.id != u.id &&
									(this.xOrder == u.xOrder+k || this.xOrder == u.xOrder-k)) {	
									superposedEdgesNumber++;
								}
							});
						}
						
					}
					
				}
			}
				
			/*for (var j = 0; j < target.length; j++) {
					
					var dist = Math.abs(target[j].xOrder - u.xOrder);
					if(dist>1 && target[j].layer == u.layer){
						for(var k=1; k<dist; k++){
							
							$.each(this.nodes, function(){
								if (this.layer == u.layer && this.xOrder == u.xOrder && this.id != u.id){
									superposedNodes.push(this);
								}
								
								if (this.layer == target[j].layer && this.xOrder == u.xOrder &&  target[j].id != target[j].id){
									superposedNodes.push(this);
								}
								
								if (this.layer == u.layer && 
									this.layer == target[j].layer && 
									this.id != target[j].id && 
									this.id != u.id &&
									//((this.xOrder > u.xOrder && this.xOrder < target[j].xOrder) || (this.xOrder < u.xOrder && this.xOrder > target[j].xOrder))
									(this.xOrder == u.xOrder+k || this.xOrder == u.xOrder-k)
									){
									
									if(superposedNodes.length > 0){
										var present = false;
										for(var n in superposedNodes){
											if(superposedNodes[n].id!=this.id){
												present=false;
											}else{
												present=true;
												break;
											}
										}
										if(!present){
											
											superposedNodes.push(this);
										}
									}else{
										//console.log(this.id)
										superposedNodes.push(this);
									}		
									
								}
							});
						//}
						
					}
					
				}

			}
		}
		
		if(superposedNodes.length!=0){
			return superposedNodes.length;
		}else{
			return 0;
		}*/
		
		return superposedEdgesNumber;
	},
	
	/**
	 return nodes superposed on edges or false if nodes are not superposed  
	 **/
	getNodesSuperposedOnEdges:function(u){

		var target = this.getInNodesTo(u);
		//var source = this.getOutNodesFrom(u);
		var superposedNodes = [];
		
		/*if (target.length > 1) {
				for (var j = 0; j < target.length; j++) {
						var dist = Math.abs(target[j].xOrder - u.xOrder);
						if(dist>=1){
							for(var k=1; k<dist; k++){
								$.each(this.nodes, function(){
									if (this.layer == u.layer && 
										this.layer == target[j].layer && 
										this.id != target[j].id && 
										this.id != u.id &&
										(this.xOrder == u.xOrder+k || this.xOrder == u.xOrder-k)) {	
										superposedNodes.push(this);
									}
								});
							}
							
						}
						
					}
			}*/
			
			for (var j = 0; j < target.length; j++) {
					
					var dist = Math.abs(target[j].xOrder - u.xOrder);
					if(dist>1 && target[j].layer == u.layer){
						for(var k=1; k<dist; k++){
							
							$.each(this.nodes, function(){
								if (this.layer == u.layer && this.xOrder == u.xOrder && this.id != u.id){
									superposedNodes.push(this);
								}
								
								if (this.layer == target[j].layer && this.xOrder == u.xOrder &&  target[j].id != target[j].id){
									superposedNodes.push(this);
								}
								
								if (this.layer == u.layer && 
									this.layer == target[j].layer && 
									this.id != target[j].id && 
									this.id != u.id &&
									//((this.xOrder > u.xOrder && this.xOrder < target[j].xOrder) || (this.xOrder < u.xOrder && this.xOrder > target[j].xOrder))
									(this.xOrder == u.xOrder+k || this.xOrder == u.xOrder-k)
									){
									
									if(superposedNodes.length > 0){
										var present = false;
										for(var n in superposedNodes){
											if(superposedNodes[n].id!=this.id){
												present=false;
											}else{
												present=true;
												break;
											}
										}
										if(!present){
											
											superposedNodes.push(this);
										}
									}else{
										//console.log(this.id)
										superposedNodes.push(this);
									}		
									
								}
							});
						//}
						
					}
					
				}

		}
		
		if(superposedNodes.length!=0){
			return superposedNodes;
		}else{
			return false;
		}
		
	},
	
	/**
	 total number of nodes superposed on nodes
	 **/
	getTotalSuperposedNodes : function() {
		var total = 0;
		var self = this;
		$.each(this.nodes, function(){
			total = total + self.getSuperposedNodesNumber(this);
		});
		return total;
	},
	
	/**
	 number of nodes superposed per node
	**/
	getTotalSuperposedNodes:function(u){
		var superposedNodesNumber=0;
		$.each(this.nodes, function(){
			if (u.layer == this.layer && u.xOrder == this.xOrder && u.id != this.id) {
				superposedNodesNumber++;

			}

		});
		
		return superposedNodesNumber;
	},
	
	/**
	 return couple of superposed nodes 
	**/
	getNodesSuperposedOnNodes:function(u){

		var superposedNodes=[];
		$.each(this.nodes, function(){
			if (u.layer == this.layer && u.xOrder == this.xOrder && this.id != v.id) {
				superposedNodes.push(u,v);
			}

		});
		
		return superposedNodes;
	},
	
	
	/**
	 some tweaks to reduce edges crossings 
	 **/
	minimizeCrossings: function(type, root){
		var self =this;
		
		if(type == "line"){
			var maxLayer = this.getAbsMaxLayer();	
			var rootLayer = root.layer;
			var totalCrossingsBefore;
			var totalCrossingsAfter;
			
			//Fist pass try move nodes in opposite layer
			$.each(this.nodes, function(){
				var layer = this.layer;
				var diff = layer - rootLayer;
				totalCrossingsBefore = self.getTotalCrossings();
				this.layer = rootLayer - diff;
				totalCrossingsAfter = self.getTotalCrossings();
				if (totalCrossingsBefore <= totalCrossingsAfter) {
					//console.log("move "+this.id+" at "+layer+ " to "+this.layer);
					this.layer = rootLayer + diff;
				
				}	
			});
			
			//second pass test all layer
			$.each(this.nodes, function(){
				var totalCrossingsBefore = self.getTotalCrossings();
				var j =0;
				
				while( j<=maxLayer){
									totalCrossingsBefore = self.getTotalCrossings();
									this.layer=this.layer+j;
									totalCrossingsAfter = self.getTotalCrossings();
									if(totalCrossingsBefore<=totalCrossingsAfter){
											this.layer=this.layer-j;
											
											totalCrossingsBefore = self.getTotalCrossings();
											this.layer=this.layer-j;
											totalCrossingsAfter = self.getTotalCrossings();
											if(totalCrossingsBefore<=totalCrossingsAfter){
												this.layer=this.layer+j;
											}
												
									}
									
									j++;
				}	
				
				self.moveNodeOnNode(this);
			});
			
		}else{
				//TODO: validate this
				var totalCrossingsBefore = this.getTotalCrossings();
				var totalCrossingsAfter = this.getTotalCrossings();
				var layer;
				
				for(n1 in this.nodes)
				{
					var u=this.nodes[n1];
					//console.log("u "+u.id);
					for (n2 in this.nodes)
					{
						var v=this.nodes[n2];
						//console.log("v "+v.id);
						var nodes = this.getNodesWithCrossingEdges(u,v);
						if(nodes[0] != undefined){
							var diff;
							for(var i=0; i<3; i++){
									totalCrossingsBefore = this.getTotalCrossings();
									layer = u.layer;//nodes[i].layer;
									diff = layer - root.layer;
									nodes[i].layer= root.layer-diff;
									totalCrossingsAfter = this.getTotalCrossings();
									if(totalCrossingsBefore<=totalCrossingsAfter){
											//console.log("move "+nodes[i].id+" at "+layer+ " to "+nodes[i].layer);
											//console.log("move "+u.id+" at "+layer+ " to "+u.layer);
											nodes[i].layer= root.layer+diff;
											
									}
							}
						}	
							var j =1;
							while(v.layer == u.layer && v.xOrder == u.xOrder && v.id !=u.id && j<100){
									totalCrossingsBefore = this.getTotalCrossings();
									v.layer=v.layer+j;
									totalCrossingsAfter = this.getTotalCrossings();
									if(totalCrossingsBefore<totalCrossingsAfter){
											v.layer=v.layer-j;
											
											totalCrossingsBefore = this.getTotalCrossings();
											v.layer=v.layer-j;
											totalCrossingsAfter = this.getTotalCrossings();
											if(totalCrossingsBefore<=totalCrossingsAfter){
												v.layer=v.layer+j;
											}	
									}
	
									j++;
							}
					}
					//move superposed nodes on nodes
					this.moveNodeOnNode(u);
				}
		}
	},

	/**
	 move (if it's possible) node when an edge is on an other one 
	 **/
	moveSuperposeEdges : function(u) {
		
		//var source = this.getOutNodesFrom(u);
		var target = this.getInNodesTo(u);
		var superposedEdges = false;
		/*if (source.length > 1) {
			for (var i = 1; i < source.length; i++) {
				if (source[i - 1].xOrder == source[i].xOrder) {
					source[i].xOrder--;
					$.each(this.nodes, function(){
						if (this.xOrder == source[i].xOrder && this.layer == source[i].layer && this.id != source[i].id) {
							source[i].xOrder = source[i].xOrder + 2;
						}
					});
					superposedEdges = true;
				}

			}

		}*/
		if (target.length > 1) {
			for (var j = 1; j < target.length; j++) {
				if (target[j - 1].xOrder == target[j].xOrder) {
					target[j].xOrder++;
					$.each(this.nodes, function(){
						if (this.xOrder == target[j].xOrder && this.layer == target[j].layer && this.id != target[j].id) {
							target[j].xOrder = target[j].xOrder - 2;
						}
					});
					superposedEdges = true;
				}
			}
		}
		
		return superposedEdges;
	},

	/**
	 move (if it's possible) node when there are on an edge
	 optimal move to reduce crossing edges and nodes superposed on nodes  
	**/
	minimizeSuperposed : function(root) {
		var self =this;
		var superposedNodes =[];
		var childrenRoot = this.getOutNodesFrom(root);
			$.each(this.nodes, function(){
					$.merge(superposedNodes, self.getNodesSuperposedOnEdges(this));
			});

			var totalCrossingsBefore;
					var totalCrossingsAfter;
					var totalSuperposedBefore;
					var totalSuperposedAfter;
					var len = superposedNodes.length;
					for (var i=0; i<len; i++){
						if(superposedNodes[i].layer>root.layer /*&& superposedNodes[i].id!=childrenRoot[0].id && superposedNodes[i].id!=childrenRoot[1].id*/ && superposedNodes[i].id!= root.id){
								totalCrossingsBefore = self.getTotalCrossings();
								totalSuperposedBefore = self.getTotalSuperposedEdges();
								superposedNodes[i].layer--;
								totalCrossingsAfter = self.getTotalCrossings();
								totalSuperposedAfter = self.getTotalSuperposedEdges();
								if(totalCrossingsBefore<totalCrossingsAfter){
										superposedNodes[i].layer+=2;
								}
								self.moveNodeOnNode(superposedNodes[i]);
								
						}else if(superposedNodes[i].layer<=root.layer /*&& superposedNodes[i].id!=childrenRoot[0].id && superposedNodes[i].id!=childrenRoot[1].id*/ && superposedNodes[i].id!= root.id){
								totalCrossingsBefore = self.getTotalCrossings();
								totalSuperposedBefore = self.getTotalSuperposedEdges();
								superposedNodes[i].layer++;
								totalCrossingsAfter = self.getTotalCrossings();
								totalSuperposedAfter = self.getTotalSuperposedEdges();
									if(totalCrossingsBefore<totalCrossingsAfter){
										superposedNodes[i].layer-=2;
								}
								self.moveNodeOnNode(superposedNodes[i]);
						
						}
					}
	

		/*var source = this.getOutNodesFrom(u);
		var superposed = false;
		var layer = u.layer;
		var xOrder = u.xOrder;
		var candidate;
		var i = 0;
		for (n in source) {
			if (layer > source[n].layer && xOrder == source[n].xOrder) {
				candidate = {
					node : u,
					source : source[n]
				};
				//console.log(candidate);
			}

		}
		if (candidate != undefined) {
			for (m in this.nodes) {

				if (this.nodes[m].xOrder == candidate.node.xOrder && candidate.source.layer < this.nodes[m].layer && this.nodes[m].layer < candidate.node.layer) {
					candidate.source.xOrder++;
					for (n in this.nodes) {
						if (this.nodes[n].xOrder == candidate.source.xOrder && this.nodes[n].layer == candidate.source.layer && this.nodes[n].id != candidate.source.id) {
							candidate.source.xOrder = candidate.source.xOrder - 2;
						}
					}
					superposed = true;

				}
				if (this.nodes[m].xOrder == candidate.node.xOrder && candidate.source.layer > this.nodes[m].layer && this.nodes[m].layer > candidate.node.layer) {
					candidate.source.xOrder++;
					for (n in this.nodes) {
						if (this.nodes[n].xOrder == candidate.source.xOrder && this.nodes[n].layer == candidate.source.layer && this.nodes[n].id != candidate.source.id) {
							candidate.source.xOrder = candidate.source.xOrder - 2;
						}
					}
					superposed = true;

				}

			}
		}*/

		//return superposed;
	},
	
	/**
	 move (if it's possible) node when there are on a node
	 optimal move to reduce crossing edges  
	**/
	moveNodeOnNode: function(u){
		
		var nodesMoved = [];
		var totalCrossingsBefore = this.getTotalCrossings();
		var totalSuperposedBefore = this.getTotalSuperposedEdges();
		$.each(this.nodes, function(){
			if (u.layer == this.layer && u.xOrder == this.xOrder && u.id != this.id) {
				nodesMoved.push(u);
				//console.log(u.id)
				u.layer++;

			}

		});
		var totalSuperposedAfter = this.getTotalSuperposedEdges();
		var totalCrossingsAfter = this.getTotalCrossings();
		if (totalCrossingsBefore < totalCrossingsAfter || totalSuperposedBefore < totalSuperposedAfter) {
			var len = nodesMoved.length;
			for (var m=0; m<len;m++) {
				//console.log(nodesMoved[m].id)
				nodesMoved[m].layer -= 2;
			}
		}

	},
	

	/* move the part of graph after the node u to the right */
	/*moveOnRight: function(u){
	 var target = this.getOutNodesFrom(u);

	 if(target[0] == undefined){
	 u.xOrder++;
	 }else{
	 for (t in target){
	 u.xOrder++;
	 u = target[t];
	 this.moveOnRight(u);
	 }
	 }
	 },*/

	/* move the part of graph after the node u to the left */
	/*moveOnLeft: function(u){
	 var target = this.getOutNodesFrom(u);

	 if(target[0] == undefined){
	 u.xOrder--;
	 }else{
	 for (t in target){
	 u.xOrder--;
	 u = target[t];
	 this.moveOnLeft(u);
	 }
	 }
	 },*/

	/**
	 test if two are nodes are connected
	**/
	isConnected : function(u, v) {
		var len = u.edges.length;
		for (var e=0; e<len;e++) {
			if (u.edges[e].source.id == v.id || u.edges[e].target.id == v.id) {
				return true;
			} else {
				return false;
			}

		}
	},

	/**
	 linearize the graph (option: type="line")
	**/
	linearize : function(removeEdgesFromDag, root) {
		
		
		//recursise function
		this.moveNodes = function(u, layer, xOrder, childrenRoot, root, diffOrder, diffLayer) {
			var target = this.getOutNodesFrom(u);
			if (target[0] == undefined && u.layer <= layer) {
				//u.xOrder = root.xOrder;
				//u.layer = root.layer+1;
				u.xOrder = 2 * xOrder - (u.xOrder + diffOrder);
				u.layer = 2 * layer - (u.layer + 2 * diffLayer);
			} else {
				var lenTarget = target.length;
				for (var t=0; t<lenTarget; t++){
					if (u.id == childrenRoot.id) {
						u.layer = u.layer + 2 * diffLayer;
						layer = u.layer;
						u.xOrder = u.xOrder + diffOrder;
						xOrder = u.xOrder;
					} else if (u.layer < layer) {
						u.layer = 2 * layer - (u.layer + 2 * diffLayer);
						u.xOrder = 2 * xOrder - (u.xOrder + diffOrder);
					}
					u = target[t];
					this.moveNodes(u, layer, xOrder, childrenRoot, root, diffOrder, diffLayer);

				}
			}
		};

		this.testStartLineLayer = function(root, childrenRoot) {
			var beforeRoot = false;
			var afterRoot = false;
			for (c in childrenRoot) {
				if (childrenRoot[c].layer == root.layer + 1) {
					afterRoot = true;
				} else if (childrenRoot[c].layer == root.layer - 1) {
					beforeRoot = true;
				}

			}
			if (beforeRoot && afterRoot) {
				return true;
			} else {
				return false;
			}
		};

		
		
			var isLinearize = false;
			var self = this;
			var childrenRoot;
			$.each(this.nodes, function(){
				if(this.id == root.id){
					childrenRoot = self.getOutNodesFrom(this);
				}
				
			});
			
			
			if(!this.testStartLineLayer(root, childrenRoot)){
				$.each(this.nodes, function(){
						isLinearize = false;
						var lenchildrenRoot=childrenRoot.length;
						for (var c=0; c<lenchildrenRoot; c++) {
							if (this.id == childrenRoot[c].id && root.xOrder != childrenRoot[c].xOrder) {
								var diffOrder = root.xOrder - childrenRoot[c].xOrder;
								var diffLayer = root.layer - childrenRoot[c].layer;
								self.moveNodes(this, this.layer, this.xOrder, childrenRoot[c], root, diffOrder, diffLayer);
								isLinearize = true;
								break;
							}
						}
					//this.moveSuperposeEdges(this.nodes[n]);
			});
		}
		//set xOrder to root xOrder when it's possible
		/*var nodesNotPossible = [];
		for (l in this.nodes) {
			var linearTarget = this.getOutNodesFrom(this.nodes[l]);
			var linearSource = this.getInNodesTo(this.nodes[l]);
			var nodesCanMove;
			//Define nodes can be move on the graphe line
			var connected;

			for (t in linearTarget) {
				for (s in linearSource) {
					connected = this.isConnected(linearTarget[t], linearSource[s]);
					numberOfConnection = this.nodeOutDegree(this.nodes[l]) + this.nodeInDegree(this.nodes[l]);
					//Move only one nodes on the line when some nodes can be move
					if (linearTarget[t].xOrder == linearSource[s].xOrder &&
					//this.nodes[l].xOrder != root.xOrder &&
					!connected /*&&
					 /*numberOfConnection<4*//*) {

						var possibleMove = false;
						//console.log(nodesNotPossible){
						//console.log(this.nodes[l].id, linearTarget[t].id, linearSource[s].id)
						//console.log("nodes can be move " +this.nodes[l].id);
						//nodesNotPossible contain all source and target of nodes already move
						if (nodesNotPossible[0] != undefined) {
							for (np in nodesNotPossible) {
								if (nodesNotPossible[np].id == linearTarget[t].id || nodesNotPossible[np].id == linearSource[s].id) {
									//the source or the target of the node is the target are the source of a node already move so it's not possible to move an other one
									possibleMove = false;
									break;
								} else {
									possibleMove = true;

								}
							}
						} else {
							possibleMove = true;
						}

						if (possibleMove) {
							nodesNotPossible.push(linearTarget[t]);
							nodesNotPossible.push(linearSource[s]);
							var forgotConnection = false;

							//console.log(removeEdgesFromDag, linearTarget[t].id, linearSource[s].id);
							//test if there are connection in edges remove when contruct dag
							for (e in removeEdgesFromDag) {
								if (removeEdgesFromDag[e].source.id == linearTarget[t].id || removeEdgesFromDag[e].target.id == linearTarget[t].id || removeEdgesFromDag[e].source.id == linearSource[s].id || removeEdgesFromDag[e].target.id == linearSource[s].id) {

									forgotConnection = true;
								} else {

									forgotConnection = false;
								}

							}

							//if there not edges use to connect the source and the target of the node, it's possible to move on line graph
							if (!forgotConnection) {
								this.nodes[l].xOrder = linearSource[s].xOrder;
							}
						}
					}

				}
			}
		}*/

		return isLinearize;
	},

	/**********Test if the graph is acyclic or not************/
	/** To test a graph for being acyclic:
	    	1 If the graph has no nodes, stop. The graph is acyclic.
	    	2 If the graph has no leaf, stop. The graph is cyclic.
	    	3 Choose a leaf of the graph. Remove this leaf and all arcs going into the leaf to get a new graph.
	    	4 Go to 1.
	**/
	isAcyclic : function() {
		//Create a copy of the graph since we are going to modify the structure by removing nodes
		var newGraph = this.copy();
		var leafs = newGraph.getLeafs();
		while (leafs.length != 0 && Object.keys(newGraph.nodes).length != 0) {
			var node = leafs.pop();
			newGraph.removeNode(node);
			leafs = newGraph.getLeafs();
		}
		if (Object.keys(newGraph.nodes).length == 0)
			return true;
		if (leafs.length == 0)
			return false;
	},
	
	/**********Get one of the Minimal FAS************/
	/** A set of edges whose removal makes the digraph acyclic is commonly known as a feedback arc set (FAS)
	The following algorithm returns T wich is a minimal FAS
	**/
	// S={}
	// Sg =new graph
	// T={}
	// for all edges e in the graph
	//		add e to Sg
	//		if Sg acyclic
	//			add e to S
	//		else
	//			remove e from Sg
	//			add e to T
	// return T
	getMinimalFAS : function() {
		var S = new Array();
		var Sg = new Graph();
		var T = new Array();
		var len=this.edges.length;
		for (var e=0; e<len;e++) {
			var edge = this.edges[e];
			Sg.addEdge(edge.source.id, edge.target.id);
			if (Sg.isAcyclic())
				S.push(edge);
			else {
				Sg.removeEdge(edge);
				T.push(edge);
			}
		}
		return T;
	},
	
	/**
	 extract the DAG 
	**/
	extractDAG : function() {
		//Create a copy of the graph since we are going to modify the structure by removing nodes
		var dag = this.copy();
		var edgesToRemove = dag.getMinimalFAS();
		dag.removeEdges(edgesToRemove);
		return dag;
	},

	/**
	 This is the heuristic proposed by Tarassov et al. in 2004.
	 It is creating a layering with a Minimum width based on DAG (if graph is not a DAG, extractDAG method has to be applied)
	 This algorithm is linear.
	 It is achieved through the min-width method with parameters W taking values {1,2,3,4} and c taking values {1,2} (as suggested by authors)
	 Then the narrowest one is chosen.
	 Finally, an improvement by promotion of vertices is applied.
	 **/
	layerAssignment : function() {
		var minWidth = Number.MAX_VALUE;
		finalC = -1;
		finalW = -1;
		for (var c = 1; c <= 2; c++) {
			for (var W = 1; W <= 4; W++) {
				var testDAG = this.copy();
				var width = testDAG.minWidthLayerAssignment(W, c);
				if (width < minWidth) {
					finalC = c;
					finalW = W;
					minWidth = width;
				}

			}
		}
		//console.log(" width "+minWidth+" chosen w "+finalW+" chosen c "+finalC);
		this.minWidthLayerAssignment(finalW, finalC);
		//Finally, an improvement by promotion of vertices is applied.
		this.vertexPromotion();
		//If there is vertex promotion it may be that there is no first layer anymore then we have to shift the layers to start at one
		this.layerShift();
	},

	/** 
	 shift layers
	 **/
	layerShift : function() {
		var minLayer = Number.MAX_VALUE;
			$.each(this.nodes, function(){
				var layer = this.layer;
				if (layer < minLayer)
					minLayer = layer;
			});
			//var shift =minLayer-1;
			// if (shift!=0)
			// {
			$.each(this.nodes, function(){
				this.layer = this.layer - minLayer;
			});
			// }
	},

	/**
	 PromoteVertex(v) by Nikolov and Tarasov 2006
	 Require: A layered DAG G = (V;E) with the layering information stored in a global
	 vertex array of integers called layering; a vertex v in V .
	 **/
	 // dummydiff=0
	 // for all u in N-G (v) do
	 // 	if layering[u] == layering[v] + 1 then
	 // 		dummydiff dummydiff+ PromoteVertex(u)
	 // 	end if
	 // end for
	 // layering[v]=layering[v] + 1
	 // dummydiff=dummydiff-N-G(v) + N+G(v)
	 // return dummydiff
	promoteVertex : function(v) {
		//console.log("Promote vertex "+v.id);
		var dummydiff = 0;
		var neigh = this.getInNodesTo(v);
		var lenNeigh = neigh.length;
		for (var x=0; x<lenNeigh; x++) {
			var u = neigh[x];
			//console.log("----"+u.id);
			if (u.layer == v.layer + 1)
				dummydiff = dummydiff + this.promoteVertex(u);
		}
		v.layer = v.layer + 1;
		dummydiff = dummydiff - this.getInNodesTo(v).length + this.getOutNodesFrom(v).length;
		//console.log("----for "+v.id+" dummydiff is "+dummydiff);
		return dummydiff;
	},

	/**
	 Vertex-Promotion Heuristic by Nikolov and Tarasov 2006
	 Require: G = (V;E) is a layered DAG; a valid layering of G is stored in a global vertex
	 array called layering.
	 **/
	 // layeringBackUp=layering
	 // repeat
	 // 	promotions=0
	 // 	for all v in V do
	 // 		if d-(v) > 0 then
	 // 			if PromoteVertex(v) < 0 then
	 // 				promotions=promotions + 1
	 // 				layeringBackUp=layering
	 // 			else
	 // 				layering=layeringBackUp
	 // 			end if
	 // 		end if
	 // 	end for
	 // until promotions = 0

	vertexPromotion : function() {

		var backUpDag = this.copy();
		var promotions = -1;
		while (promotions != 0) {
			promotions = 0;
			for (x in this.nodes) {
				var v = this.nodes[x];
				//console.log("***for "+v.id);
				if (this.getInNodesTo(v).length > 0) {
					// console.log("Promote for "+v.id+" is "+this.promoteVertex(v));
					// console.log("Promote for "+v.id+" is "+this.promoteVertex(v));
					// console.log(this.promoteVertex(v)<0);
					//console.log("Compute promotion for "+v.id);
					var promotionVal = this.promoteVertex(v);
					//console.log("Promote for "+v.id+" is "+promotionVal);
					var promotion = (promotionVal < 0);
					//console.log("there is a promotion "+promotion);
					//if(this.promoteVertex(v)<0)
					if (promotion) {
						//console.log("******for "+v.id+" there is an improvement");
						promotions = promotions + 1;
						backUpDag = this.copy();
					} else {
						for (n in backUpDag.nodes) {
							//console.log(this.nodes[n].layer+"---"+backUpDag.nodes[n].layer);
							this.nodes[n].layer = backUpDag.nodes[n].layer;
						}
					}
				} else
					console.log("--- in degree is below zero");
			}
		}
	},

	/**
	 minWidthLayerAssignment algo
	**/
	//U={}; Z={}
	//currentLayer=1; widthCurrent=0; widthUp=0
	//while U != V do
	// 	Select vertex v in V\U with N+G(v) C or equal to Z and ConditionSelect (node with the highest outdegree)
	// 	if v has been selected then
	//		 Assign v to the layer with a number currentLayer
	// 		U=U union {v}
	// 		widthCurrent=widthCurrent-d+(v) + 1
	//		 widthUp=widthUp + d-(v)
	//	end if
	// 	if no vertex has been selected OR ConditionGoUp then
	// 		currentLayer=currentLayer + 1
	//		 Z=Z union U
	// 		widthCurrent=widthUp
	// 		widthUp=0
	// 	end if
	//end while
	minWidthLayerAssignment : function(W, c) {
		var maxWidth = 0;
		var V = new Array();
		$.each(this.nodes, function(){
			V.push(this);
			this.layer = -1;
		});
		var U = new Array();
		var Z = new Array();
		var currentLayer = 1;
		var widthCurrent = 0;
		var widthUp = 0;

		while (!($(U).not(V).length == 0 && $(V).not(U).length == 0))//test if U!=V using JQuery
		{
			//Select vertex v in V\U with N+G(v) inclued or equal to Z and with maximum outdegree
			//build V\U
			var VminusU = $.grep(V, function(x) {
				return $.inArray(x, U) < 0
			});
			//get the nodes with N+G(v) inclued or equal to Z
			var candidates = new Array();
			var lenVminusU =VminusU.length;
			for (var i=0; i<lenVminusU; i++) {
				var Nplusn = this.getOutNodesFrom(VminusU[i]);
				if ($(Nplusn).not(Z).length == 0)
					candidates.push(VminusU[i]);
			}
			var candidateFound = false;
			var vOutDegree = Number.MAX_VALUE;
			if (candidates.length != 0) {
				candidateFound = true;
				//select the highest degree node in V\U
				var v = this.quickDegreeSort(candidates).pop();
				v.layer = currentLayer;
				U.push(v);
				vOutDegree = this.getOutNodesFrom(v);
				widthCurrent = widthCurrent - this.getOutNodesFrom(v).length + 1;
				widthUp = widthUp + this.getInNodesTo(v).length;
			}
			if (!candidateFound || (widthCurrent >= W && vOutDegree < 1) || widthUp >= c * W) {
				currentLayer = currentLayer + 1;
				var lenU= U.length;
				for (var n=0; n<lenU;n++) {
					Z.push(U[n]);
				}
				if (widthCurrent > maxWidth)
					maxWidth = widthCurrent;
				widthCurrent = widthUp;
				widthUp = 0;
			}
		}
		return maxWidth;
	},

	/**  
	 return array with targets nodes 
	 **/
	getOutNodesFrom : function(node) {
		var out = new Array();
		var len= node.edges.length;
		for (var e=0; e<len;e++) {
			if (node.edges[e].target.id != node.id)
				out.push(node.edges[e].target);
		}
		return out;
	},

	/**  
	 return array with source nodes 
	 **/
	getInNodesTo : function(node) {
		var inNodes = new Array();
		var len= node.edges.length;
		for (var e=0; e<len;e++) {
			if (node.edges[e].source.id != node.id)
				inNodes.push(node.edges[e].source);
		}
		return inNodes;
	},

	/**********Get strongly connected components************/
	/** Tarjan algorithm to detect all strongly connected components in a graph.
	The method will return an array of arrays containing all the strongly connected component nodes.
	**/

	// algorithm tarjan is
	//   input: graph G = (V, E)
	//   output: set of strongly connected components (sets of vertices)
	//   index := 0
	//   S := empty
	//   for each v in V do
	//      if (v.index is undefined) then
	//       strongconnect(v)
	//     end if
	//   end for
	//   function strongconnect(v)
	//     // Set the depth index for v to the smallest unused index
	//     v.index := index
	//     v.lowlink := index
	//     index := index + 1
	//     S.push(v)

	//     // Consider successors of v
	//      for each (v, w) in E do
	//        if (w.index is undefined) then
	//          // Successor w has not yet been visited; recurse on it
	//         strongconnect(w)
	//         v.lowlink  := min(v.lowlink, w.lowlink)
	//       else if (w is in S) then
	//          // Successor w is in stack S and hence in the current SCC
	//          v.lowlink  := min(v.lowlink, w.index)
	//       end if
	//     end for

	//     // If v is a root node, pop the stack and generate an SCC
	//     if (v.lowlink = v.index) then
	//       start a new strongly connected component
	//       repeat
	//         w := S.pop()
	//         add w to current strongly connected component
	//       until (w = v)
	//       output the current strongly connected component
	//     end if
	//   end function
	getStronglyConnectedComponents : function() {
		// 	if(graph==null)
		// 		graph=this.createGraph(false);
		var S = new Array();
		var index = 0;
		var scc = new Array();
		var self=this;
		$.each(this.nodes, function(){
			this.index = undefined;
			this.lowlink = Number.MAX_VALUE;
		});
		$.each(this.nodes, function(){
			if (this.index == undefined)
				self.strongconnect(this, index, S, scc);
		});
		return scc;
	},
	strongconnect : function(v, index, S, scc) {
		v.index = index;
		v.lowlink = index;
		index = index + 1;
		S.push(v);
		var lenVedges= v.edges.length;
		for (var e=0; e<lenVedges;e++) {
			var w = v.edges[e].target;
			if (w.id != v.id) {
				if (w.index == undefined) {
					this.strongconnect(w, index, S, scc);
					v.lowlink = Math.min(v.lowlink, w.lowlink);
				} else {
					if (S.indexOf(w) != undefined) {
						v.lowlink = Math.min(v.lowlink, w.lowlink);
					}
				}
			}
		}
		if (v.lowlink == v.index) {
			var comp = new Array();
			var w = S.pop();
			while (w.id != v.id) {
				comp.push(w);
				var w = S.pop();
			}
			comp.push(w);
			scc.push(comp);
		}
	},

	/**
	Quick Sort:
		1. Select some random value from the array, the median.
		2. Divide the array in three smaller arrays according to the elements being less, equal or greater than the median.
		3. Recursively sort the array containg the elements less than the median and the one containing elements greater than the median.
		4. Concatenate the three arrays (less, equal and greater).
		5. One or no element is always sorted.
	 **/
	//TODO: This could be implemented more efficiently by using only one array object and several pointers.
	quickDegreeSort : function(nodes) {
		/* recursion anchor: one element is always sorted */
		if (nodes.length <= 1)
			return nodes;
		/* randomly selecting some value */
		var midIndex = Math.floor(nodes.length / 2);
		var median = nodes[midIndex];
		var arr1 = [], arr2 = [], arr3 = [];
		for (var i in nodes) {
			this.nodeOutDegree(nodes[i]) < this.nodeOutDegree(median) && arr1.push(nodes[i]);
			this.nodeOutDegree(nodes[i]) == this.nodeOutDegree(median) && arr2.push(nodes[i]);
			this.nodeOutDegree(nodes[i]) > this.nodeOutDegree(median) && arr3.push(nodes[i]);
		}
		/* recursive sorting and assembling final result */
		return this.quickDegreeSort(arr1).concat(arr2).concat(this.quickDegreeSort(arr3));
	},
};

/*
 *Node
 */
Graph.Node = function(id, node) {
	node = node || {};
	node.id = id;
	node.edges = [];
	node.hide = function() {
		this.hidden = true;
		this.shape && this.shape.hide();
		/* FIXME this is representation specific code and should be elsewhere */
		var len= this.edges.length;
		for (var i=0; i<len;i++)(this.edges[i].source.id == id || this.edges[i].target == id) && this.edges[i].hide && this.edges[i].hide();
	};
	node.show = function() {
		this.hidden = false;
		this.shape && this.shape.show();
		var len= this.edges.length;
		for (var i=0; i<len;i++)(this.edges[i].source.id == id || this.edges[i].target == id) && this.edges[i].show && this.edges[i].show();
	};
	return node;
};

Graph.Node.prototype = {
};

/**
 A simple binary min-heap serving as a priority queue
 - takes an array as the input, with elements having a key property
 - elements will look like this:
 {
 key: "... key property ...",
 value: "... element content ..."
 }
 - provides insert(), min(), extractMin() and heapify()
 - example usage (e.g. via the Firebug or Chromium console):
 var x = {foo: 20, hui: "bla"};
 var a = new BinaryMinHeap([x,{foo:3},{foo:10},{foo:20},{foo:30},{foo:6},{foo:1},{foo:3}],"foo");
 console.log(a.extractMin());
 console.log(a.extractMin());
 x.foo = 0; // update key
 a.heapify(); // call this always after having a key updated
 console.log(a.extractMin());
 console.log(a.extractMin());
 - can also be used on a simple array, like [9,7,8,5]
 **/
function BinaryMinHeap(array, key) {

	/* Binary tree stored in an array, no need for a complicated data structure */
	var tree = [];

	var key = key || 'key';

	/* Calculate the index of the parent or a child */
	var parent = function(index) {
		return Math.floor((index - 1) / 2);
	};
	var right = function(index) {
		return 2 * index + 2;
	};
	var left = function(index) {
		return 2 * index + 1;
	};

	/* Helper function to swap elements with their parent
	 as long as the parent is bigger */
	function bubble_up(i) {
		var p = parent(i);
		while ((p >= 0) && (tree[i][key] < tree[p][key])) {
			/* swap with parent */
			tree[i] = tree.splice(p, 1, tree[i])[0];
			/* go up one level */
			i = p;
			p = parent(i);
		}
	}

	/* Helper function to swap elements with the smaller of their children
	 as long as there is one */
	function bubble_down(i) {
		var l = left(i);
		var r = right(i);

		/* as long as there are smaller children */
		while (tree[l] && (tree[i][key] > tree[l][key]) || tree[r] && (tree[i][key] > tree[r][key])) {

			/* find smaller child */
			var child = tree[l] ? tree[r] ? tree[l][key] > tree[r][key] ? r : l : l : l;

			/* swap with smaller child with current element */
			tree[i] = tree.splice(child, 1, tree[i])[0];

			/* go up one level */
			i = child;
			l = left(i);
			r = right(i);
		}
	}

	/* Insert a new element with respect to the heap property
	 1. Insert the element at the end
	 2. Bubble it up until it is smaller than its parent */
	this.insert = function(element) {

		/* make sure there's a key property */
		(element[key] == undefined) && ( element = {
			key : element
		});

		/* insert element at the end */
		tree.push(element);

		/* bubble up the element */
		bubble_up(tree.length - 1);
	};

	/* Only show us the minimum */
	this.min = function() {
		return tree.length == 1 ? undefined : tree[0];
	};

	/* Return and remove the minimum
	 1. Take the root as the minimum that we are looking for
	 2. Move the last element to the root (thereby deleting the root)
	 3. Compare the new root with both of its children, swap it with the
	 smaller child and then check again from there (bubble down)
	 */
	this.extractMin = function() {
		var result = this.min();

		/* move the last element to the root or empty the tree completely */
		/* bubble down the new root if necessary */
		(tree.length == 1) && ( tree = []) || (tree[0] = tree.pop()) && bubble_down(0);

		return result;
	};

	/* currently unused, TODO implement */
	this.changeKey = function(index, key) {
		throw "function not implemented";
	};

	this.heapify = function() {
		for (var start = Math.floor((tree.length - 2) / 2); start >= 0; start--) {
			bubble_down(start);
		}
	};

	/* insert the input elements one by one only when we don't have a key property (TODO can be done more elegant) */
	for (i in (array || []))
	this.insert(array[i]);
}

// 	/**
// 	Extract a subnetwork based on lightest path length between each pair of selected nodes
// 	it returns a graph where nodes have a subnet attribute telling wether they are in subnet or not
// 	**/
// 	extractSubNetwork: function(){

// 		var cy = $("#cy").cytoscape('get');
// 		// Should be a parameter
// 		var nodeSelection = cy.$('node:selected');
// 		if (nodeSelection.length<2)
// 			Ext.Msg.alert("Warning","At least two nodes have to be selected.");
// 		else
// 		{
// 			//Map containing nodes,index values
// 			var selectedNodesIndex={};
// 			for(i=0;i<nodeSelection.length;i++)
// 			{
// 				selectedNodesIndex[nodeSelection[i].data("id")]=i;
// 			}
// 			//Create a matrix containing all the paths
// 			var paths=[];
// 			//for each pair of selected nodes, create an entry in the path matrix
// 			for(i=0;i<nodeSelection.length;i++)
// 			{
// 				paths[i]=[];
// 				for(j=0;j<nodeSelection.length;j++)
// 					paths[i][j]=null;
// 			}
// 			//create the graph structure based on the displayed network
// 			//This graph will contain also backward reactions
// 			var graph=this.createGraph(true);
// 		  	for(var n in graph.nodes)
// 			{
// 		    	graph.nodes[n].inSubNet=false;
// 		    }
// 		    for (var i=0;i<graph.edges.length;i++)
// 		    {
// 		    	graph.edges[i].inSubNet=false;
// 		    }
// 			//For each node in the selection
// 			//	Compute its lightest path to other nodes
// 			for(var i = 0; i < nodeSelection.length; i++ )
// 				{
// 					var nodeI = nodeSelection[i];
// 					var selectedNodeID=nodeI.data("id");
// 					//Since while searching for a path we may find another one
// 	  				//We are going to keep the path already found
// 	  				var jWithPath=new Array();
// 					for(var j=0;j<nodeSelection.length;j++)
// 	  				{
// 	  					if(j!=i)
// 	  					{
// 	  						//if the paths is not already computed
// 	  						if(paths[i][j]==null)
// 	  						{
// 	  							//If we go through a reaction, we want to avoid going through its back version
// 								var reactionUsed=new Array();
// 	  							//if not, do the computation
// 		  						var nodeJ=nodeSelection[j];
// 		  						var graphNodeJ=nodeJ.data("id");
// 		  						for(var n in graph.nodes)
// 				    			{
// 		    						graph.nodes[n].distance = Infinity;
// 		    						graph.nodes[n].predecessor = null;
// 		    						graph.nodes[n].optimized = false;
// 		    					}
// 				    			var source=graph.nodes[selectedNodeID];
// 				    			source.distance = 0;
// 				   				/* set of unoptimized nodes, sorted by their distance (but a Fibonacci heap
// 				       			would be better) */
// 				    			var q = new BinaryMinHeap(graph.nodes, "distance");
// 								//pointer to the node in focus
// 							    var node;
// 							    /* get the node with the smallest distance
// 				       			as long as we have unoptimized nodes. q.min() can have O(log n). */
// 				       			var targetReached=false;
// 				       			while((q.min() != undefined) &&(!targetReached))
// 							    {
// 							        /* remove the latest */
// 							        node = q.extractMin();
// 				        			node.optimized = true;
// 							        for(e in node.edges)
// 							        {
// 								    	var other = node.edges[e].target;
// 							            if(other.optimized)
// 							                continue;
// 							            /* look for an alternative route */
// 							            //var alt = node.distance + node.edges[e].weight;
// 							            /*Implementation of the lightest path*/
// 							            /*will penalize high degree nodes*/
// 							            var alt = node.distance + (node.edges.length)*(node.edges.length);
// 							            /* update distance and route if a better one has been found */
// 							            if ((alt < other.distance) && reactionUsed.indexOf(this.getReactionIdWithoutBack(other.id))==-1)
// 							            {
// 							            	/*Add the reaction to the visited ones*/
// 							            	reactionUsed.push(this.getReactionIdWithoutBack(other.id));
// 							                /* update distance of neighbour */
// 							                other.distance = alt;
// 							                /* update priority queue */
// 							                q.heapify();
// 							                /* update path */
// 							                other.predecessor = node;
// 							            }
// 							            if (other.id==nodeJ.data("id"))
// 							            {
// 							            	//if the target is reached then we will stop
// 							            	targetReached=true;
// 							            	paths[i][j]=this.getPathBasedOnPredecessors(graph,nodeJ.data("id"));
// 							            }
// 							            //if the node is a node in the selection list
// 							            //if there is no corresponding path
// 							            //we add the paths from i to it!
// 							            else
// 							            {
// 							            	var nodeIndex=selectedNodesIndex[other.id];
// 								            if ((nodeIndex!=undefined) && (paths[i][nodeIndex]==null))
// 								            {
// 								            	paths[i][nodeIndex]=this.getPathBasedOnPredecessors(graph,other.id);
// 								            }
// 							            }
// 									}
// 								}

// 							}
// 	  					}
// 	  				}
// 	  			}
// 	  		return graph;
// 		}
// 	},

// /**********************************************/
// 	/******************************************/
// 	//BINDING BETWEEN CYTOSCAPE AND GRAPH
// 	/******************************************/
// /**********************************************/

// 	*
// 	Function that will create the graph on which we are going to do the computation
// 	This function create the graph based on the displayed cytoscape network
// 	If reversible=true
// 		For each reversible reactions, we are going to add the backward version of the reaction
// 	else
// 		The graph structure fits the drawing.
// 	*
// 	// createGraph:function(reversible){
// 	// 	var cy = $("#cy").cytoscape('get');
// 	// 	//Graph object coming from the Dracula library
// 	// 	var graph=new Graph();
// 	// 	var eles=cy.elements("node[biologicalType='reaction']");
//  //      	for(var i=0;i<eles.length;i++)
//  //      		{
//  //      			var reaction=eles[i];
//  //      			//add the reaction
//  //      			graph.addNode(reaction.data("id"));
//  //      			//add the backward reaction if reversible
//  //      			var idBack=reaction.data("id")+"_back";
//  //      			if(reversible)
// 	//       			if(reaction.data("reversible"))
// 	// 	      			graph.addNode(idBack);
//  //      			//for each product, add the edge
//  //      			var products=this.getDirectedNeighborsOUT(reaction);
//  //      			for(var j=0;j<products.length;j++)
//  //      				{
//  //      					graph.addEdge(reaction.data("id"),products[j].data("id"));
//  //      					if(reaction.data("reversible"))
//  //      						if(reversible)
// 	//       						graph.addEdge(products[j].data("id"),idBack);
//  //      				}
//  //      			//for each substrate, add the edge
//  //      			var substrates=this.getDirectedNeighborsIN(reaction);
//  //      			for(var j=0;j<substrates.length;j++)
//  //      				{
//  //      					graph.addEdge(substrates[j].data("id"),reaction.data("id"));
//  //      					if(reversible)
// 	//       					if(reaction.data("reversible"))
//  //    	  						graph.addEdge(idBack,substrates[j].data("id"));
//  //      				}

//  //      		}
// 	// 	return graph;
// 	// },

// /**********************************************************************************/
// 	/******************************************************************************/
// 	//GRAPH STRUCTURE AND ALGORITHMS ---- SHOULD BE ADDED TO THE GRAPH LIBRARY !
// 	/******************************************************************************/
// /**********************************************************************************/

// 	/*************************
// 	/ In order to speed up the computation a graph structure is used.
// 	/ A graph is created corresponding to the metabolic network as displayed in Cytoscape
// 	/ Algorithms are applied to this graph and then results are displayed in Cytoscape window

// 	/ For the graph objects we develop a set of graph algorithms:
// 	/ - Get one of the minimal Feedback arc sets
// 	/ - Get strongly connected components
// 	/ - Get leaf nodes in a graph
// 	/ - Test if the graph is acyclic or not
// 	/**********************
// 	*/

// //IN PROGRESS !
// //Function to extract FAS with selected compounds as input and outputs
// getMinimalFASstory:function(graph,selectedNodesID)
// 	{
// 		//console.log("Graph is acyclic "+this.isAcyclic(graph));
// 		var S=new Array();
// 		var Sg=new Graph();
// 		var T=new Array();
// 		//Order the edge that can be removed
// 		//Edge connecting higly connected nodes first
// 		var maxValue=graph.edges.length;
// 		for(e in graph.edges)
// 		{
// 			var edge=graph.edges[e];
// 			var weight=0;
// 			if(selectedNodesID.length>0)
// 			{
// 				if(selectedNodesID.indexOf(edge.source.id)==-1)
// 					weight=weight+maxValue;
// 				if(selectedNodesID.indexOf(edge.target.id)==-1)
// 					weight=weight+maxValue;
// 			}
// 			weight=weight-Math.max(edge.source.edges.length,weight+edge.target.edges.length);
// //			weight=weight+edge.target.edges.length;
// 			edge.weight=weight;
// 		}

// 		graph.edges.sort(function(obj1, obj2) {
// 			return obj1.weight - obj2.weight;
// 			});

// 		for(e in graph.edges)
// 		{
// 			var edge=graph.edges[e];
// 			Sg.addEdge(edge.source.id,edge.target.id);
// 			if(this.isAcyclic(Sg))
// 				S.push(edge);
// 			else
// 			{
// 				Sg.removeEdge(edge);
// 				T.push(edge);
// 			}
// 		}
// 		return T;
// 	},
// });
