/**
 * From <a href="http://stackoverflow.com/questions/119441/highlight-a-word-with-jquery">stackoverflow.com<a> 
 * Allow to highlight a motif in a text
 * @module highlight
 * 
 * @requires <a href='http://blog.jquery.com/2011/09/12/jquery-1-6-4-released/'>jQuery 1.6.4+</a>
 * @dependency <script language="JavaScript" type="text/javascript" src="../biojs/dependencies/jquery/jquery-1.4.2.min.js"></script>
 * 
 * @example
 * 	$(selector).highlight(modif);
 */
(function($) {
"use strict";
		/**
		 * @method highlight
 		 * @param {Object} $el selector contained string to highlight
 		 * @param {Strion} word the modif to highlight
 		 * @private
 		 */
		function highlight($el, word) {
			var text = $el.text();
			$el.empty().text(text);
			var motif = $el.html();
			
			word = word.toUpperCase();

			var posRead = 0; 
			var len = word.length;
			
			var match = motif.match(new RegExp(word, 'g'));
			
			if(match!= undefined){
				var rep = match.length;
				
				var arrayStart = []; 
				var arrayStop =  [];
				//Find positions for each motif
				for(var i=0; i<rep; i++){				
				    var pos = motif.indexOf(word, posRead);
					arrayStart[i]= pos+1;
					arrayStop[i] = pos+len;
					posRead = pos+len;				
				}
				
				motif = motif.replace(new RegExp(word, 'g'), '<span class="hl">' + word + '</span>');
				$el.html(motif);
				$el.data("str", motif);
				$el.data("repeat", rep);
				$el.data("start", arrayStart);
				$el.data("stop", arrayStop);
			}
			

		}

		/**
		 * Add span marker with hl class around the word that must be highlight
		 * @class highlight
		 * @param {String} word the modif to highlight
		 * @constructor
		 */
		$.fn.highlight = function(word) {
			return this.each(function() {
				highlight($(this), word);
			});
		};

}(jQuery)); 